// CONSTANTS

// number of monsters a blight can support at the start
const nether_startmonsters := 1;
const nether_maxmonsters := 10;

// maximum netherblight power level
// 5 and 6 are insanely nasty.
const nether_maxpower := 5;

Const UOBJ_CORRUPTION := 0x706c;
Const UOBJ_MAIN_CORRUPTION := 0x706d;
Const UOBJ_UBER_CORRUPTION := 0x7088;

//this gets multiplied by the power - a very powerful blight has a long
//reach.
const nether_spawndistance := 250;

Const nether_cfg := "superblight_spawn";

const nether_minx := 0;
const nether_miny := 0;
const nether_maxx := 5200;
const nether_maxy := 3100;

// anchor constants for moved creatures
const nether_ds := 10;
const nether_ps := 10;


// CHANCES
// number of seconds until next event
// value + randint(value)


// chance a blight gets it's monster cap raised.
const nether_chancemoremonsters := 3600;

// chance that number of child blights improves
/* 
  this number is used slightly differently from the others.  It is multiplied
by the current number of children and the order (distance from the parent).  
So children will be spit out fairly quickly for a bit, but then the rate will 
slow down. */
// * offspring * offspring * order
const nether_chancemorechildren := 7200;

// chance a child blight spawns from a parent
const nether_chancespreadblight := 3600;

// delaying factor on blight spreading
const nether_spreaddelay := 0;

// chance a blight gets stronger.
// * order
const nether_chanceblightstronger := 21600;

// chance to kill monsters that are weaker than the current strength
const nether_chanceslaytheweak := 10800;

// chance to spawn a new monster
const nether_newmon := 300;

// delay to spawn a nether crystal
const nether_crystal := 3600;



/* Ap's Notes:
   A shout out to the unknown soul who wrote the druid blight code.  Much was
   appropriated, and much thanks is given.
   
   These blights make nastier monsters, and they spread.  However, taking out
   a parent blight will destroy all child blights, so we have a game of "hunt
   the core of destruction".
*/


/*
  This function writes the blight's name, "nether blight".  The closer it
is to being the parent blight, the more likely the letters are to be capital.
The parent blight should *always* be "NETHER BLIGHT", second order blights will
be about half capital letters, and far out blights will be mostly lowercase.
*/

function blightname(order)
	var r := "";
	
	foreach l in {"n","e","t","h","e","r"," ","b","l","i","g","h","t"}
		if (r != " ")
			r := r + vletter(order,upper(l),l);
		else
			r := r + " ";
		endif
	endforeach
	
	return r;
endfunction

function vletter(order,lar,sma)
	if (RandomInt(order) == 0)
		return lar;
	else
		return sma;
	endif
endfunction

function destroynblight(nb)	
	var ttd := {};
	var x;

	storeorphanedmonsters(monsterlist);

	foreach nbs in offspring
		x := SystemFindObjectBySerial (nbs);
		if (x)
			setobjproperty(x,"destroy",1);
		endif
	endforeach
	
//	sleep(13);
	
	foreach square in ListItemsNearLocationOfType (nb.x, nb.y, nb.z, 5, UOBJ_CORRUPTION)
		if ( getobjproperty(square,"attachedto") == nb.serial )
			ttd.append(square);
		endif
	endforeach

	ttd.append(nb);

	while(len(ttd))
		x := random(len(ttd))+1;
		PlayStationaryEffect( ttd[x].x, ttd[x].y, ttd[x].z, 0x3709, 1,30,1);
		sleepms(250);
		DestroyItem(ttd[x]);
		ttd.erase(x);
	endwhile
endfunction

function landscore(nb)
	var r:=0;
	var x,y,z;

	for i := 1 to 50
		x:=nb.x - 10 + RandomInt(20);
		y:=nb.y - 10 + RandomInt(20);
		z := GetMapInfo(x,y).z;
		
		if (CheckLosAt( nb , x, y, z ))
			if ( abs( (GetMapInfo(x,y).z) - (GetMapInfo(nb.x,nb.y).z) ) < 4)
				if (GetMapInfo(x,y).landtile < 100)
					r:=r+1;
				endif
			endif
		endif
				
		sleepms(1);
	endfor
	
	return r;
endfunction

function createnblight (x,y,order,par)
	var o := CreateItemAtLocation( x, y, GetMapInfo(x,y).z, 0x7088, 1 );
	setobjproperty(o,"order",order);
	setobjproperty(o,"parent",par.serial);
	
	return o.serial;
endfunction

/*
All our times have come
Here but now they're gone
Seasons don't fear the reaper
Nor do the wind, the sun or the rain
We can be like they are
*/

function perpetuate()
	var mnbs := GetGlobalProperty("rootblight");
	var x,y;
	
	while (0==0)
		if (!mnbs)
			x := Random(nether_maxx - nether_minx) + nether_minx;
			y := Random(nether_maxy - nether_miny) + nether_miny;
			mnbs := createnblight(x,y,1,0);
		endif

		sleep(60);
				
		if (!(SystemFindObjectBySerial(mnbs)))
			mnbs := 0;
		endif
			
		sleep(60);
	endwhile
endfunction

/* from an array of serials, remove the ones we can't find. return new array.*/
function cleanlist(x)
	for i := 1 to len(x)
		if (!SystemFindObjectBySerial(x[i]))
			x.erase(i);
		endif
	endfor
	
	return x;
endfunction

function storeorphanedmonsters(monsterlist)
	cleanlist(monsterlist);
	set_critical(1);
	var strayblightmonsters := GetGlobalProperty("strayblightmonsters");
	foreach x in monsterlist
		strayblightmonsters.append(x);
	endforeach

	SetGlobalProperty("strayblightmonsters",strayblightmonsters);
	set_critical(0);
endfunction

function apop(x)
	var ret := 0;
	
	if (len(x) == 0)
		return 0;
	endif
	
	ret := x[1];
	x.erase(1);
	
	return ret;
endfunction

function getorphanedmonster(mct)
	var ret := 0;

	set_critical(1);
	var strayblightmonsters := GetGlobalProperty("strayblightmonsters");
	if (!strayblightmonsters)
		strayblightmonsters := {};
	endif
	
	var n := 0;
	var o;

	while ( len(strayblightmonsters) > 0 )
//		printtextabove(mct,"*" + strayblightmonsters + "*");
		
		n:=strayblightmonsters[1];
		strayblightmonsters.erase(1);
		
		o := SystemFindObjectBySerial(n);
		if (o)
			SetGlobalProperty("strayblightmonsters",strayblightmonsters);
			set_critical(0);
			return o;
		endif
	endwhile
	
	SetGlobalProperty("strayblightmonsters",strayblightmonsters);
	set_critical(0);
	return 0;
endfunction

function getdelay_more_monsters(ctime,order,monsters)
	return ctime + (nether_chancemoremonsters + Random(nether_chancemoremonsters)) * ((order) + (monsters));
endfunction

function getdelay_spawn_monsters(ctime)
	return ctime + nether_newmon + Random(nether_newmon);
endfunction

function getdelay_more_blights(ctime,order,offspring)
	return ctime + (nether_chancemorechildren + Random(nether_chancemorechildren)) * (order + (offspring * offspring) + nether_spreaddelay);
endfunction

function getdelay_spawn_blights(ctime)
	return ctime + nether_chancespreadblight + Random(nether_chancespreadblight);
endfunction


function getdelay_stronger(ctime,order)
	return ctime + (nether_chanceblightstronger + Random(nether_chanceblightstronger)) * (order);
endfunction

function getdelay_slaytheweak(ctime)
	return ctime + nether_chanceslaytheweak + Random(nether_chanceslaytheweak);
endfunction

function getdelay_nethercrystal(ctime)
	return ctime + nether_crystal + Random(nether_crystal);
endfunction