use os;
use cfgfile;
use uo;

const EVENT_ADD_POISON			:= 0x1;
const EVENT_CURE_POISON			:= 0x2;
const EVENT_CURE_SPECIFIC_POISON	:= 0x3;
const EVENT_DEAD			:= 0x4;

//
// ListPoisonsByType(mobile, type)
// Returns an array of strings listing poisons names
// running on the targetted mobile that are of the type 
// passed.
//
function ListPoisonsByType(mobile, the_type)
	var p_list := array;
	var temp_list := GetObjProperty(mobile, "Poisons");
	var temp := temp_list.keys();
	foreach poison_name in (temp_list.keys())
		if ( lower(CfgInfo(poison_name).Type) == lower(the_type) )
			p_list.append(poison_name);
		endif
		sleepms(2);
	endforeach
	
	return p_list;
endfunction

//
// GetAllPoisons(mobile)
// Returns an array of strings listing the poison names
// running on the mobile passed.
//
function GetAllPoisons(mobile)
	var p_list := array;
	var temp_list := GetObjProperty(mobile, "Poisons");
	var temp := temp_list.keys();
	foreach poison_name in (temp_list.keys())
		p_list.append(poison_name);
		sleepms(2);
	endforeach
	
	return p_list;
endfunction

//
// AddPoison(mobile, poison_name, amount)
// Adds a poison to the mobile for 'amount' seconds.
//
function AddPoison(mobile, poison_name, amount)
	var event := struct;
	event.+ type;
	event.+ name;
	event.+ amount;
	
	event.type := EVENT_ADD_POISON;
	event.name := poison_name;
	event.amount := amount;
	
	GetScript(mobile).SendEvent(event);
endfunction

//
// CurePoison(mobile, amount)
// Cures 'amount' of seconds of poison from
// the mobile. This will only cure poisons that
// are NOT set to 'NoCure 1'.
//
function CurePoisonByAmount(mobile, amount)
	var event := struct;
	event.+ type;
	event.+ name;
	event.+ amount;
	
	event.type := EVENT_CURE_POISON;
	event.amount := amount;
	
	GetScript(mobile).SendEvent(event);
endfunction

// 
// CureSpecific(mobile, poison_name, amount)
// Cures 'amount' seconds of a specific type of poison.
// Ignores the 'NoCure' setting. 
//
function CureSpecific(mobile, poison_name, amount)
	var event := struct;
	event.+ type;
	event.+ name;
	event.+ amount;
	
	event.type := EVENT_CURE_SPECIFIC_POISON;
	event.name := poison_name;
	event.amount := amount;
	
	GetScript(mobile).SendEvent(event);
endfunction

//
// GetScript(mobile)
// Returns the process of the poison watcher.
// If the poison watcher is not running, it will 
// start it up on the mobile unless no_start is set to
// a non-zero value.
//
function GetScript(mobile, no_start:=0)
	var poison_pid := CInt(GetObjProperty(mobile, "#PoisonPID"));
	var poison_watcher := GetProcess(poison_pid);
	if ( !poison_watcher && !no_start ) 
		//How to start it on an npc!
		poison_watcher := start_script(":poisons:poisonwatcher", {mobile});
		
		if ( poison_watcher.errortext )
		//	PrintTextAbove(mobile, "Poison Watcher GetScript() " + poison_watcher.errortext);
		endif
	endif
	
	return poison_watcher;
endfunction

//
// CfgInfo(poison_name)
// Returns config information about the poison name passed.
//
function CfgInfo(poison_name)
	return ReadConfigFile(":poisons:poisons")[poison_name];	
endfunction
