include "include/client";
include "include/math";
include "include/yesno";
include "include/attributes";
include "include/skills";

const ORDERGUARDTYPE := 1;
const CHAOSGUARDTYPE := 2;
const CHAOSSHIELDTYPE := 0x1bc3;
const ORDERSHIELDTYPE := 0x1bc4;

function AddVirtue (who, amount := 1, nomsg:=0)
	if (!amount)
		amount := 1;
	endif

	var yourvirtue := CINT(GetObjProperty (who, "virtue"));
	if (!yourvirtue)
		yourvirtue := 0;
	elseif (yourvirtue == 50000)
		return;
	endif

	var desc;
	case (abs(amount))
		1: desc := " a tiny amount of ";
		2: desc := " a very small amount of ";
		3: desc := " a small amount of ";
		4: desc := " some ";
		5: desc := " a good amount of ";
		6: desc := " a lot of ";
		7: desc := " a great amount of ";
		default: desc := " a massive amount of ";
	endcase

	yourvirtue := yourvirtue + amount;

	if (!nomsg)
		if (amount > 0)
			SendSysMessage(who,"You have gained" + desc + "Virtue", FONT_NORMAL, 69);
		else
			SendSysMessage(who,"You have lost" + desc + "Virtue", FONT_NORMAL, 40);		
		endif
	endif
	SetObjProperty (who, "virtue", yourvirtue);

	var guardtype := GetObjProperty(who,"guardtype");
	if (!guardtype)
		SetNonVGTitle( who );
	elseif ( guardtype == CHAOSGUARDTYPE )
		SetMyTitle(who, yourvirtue);
	elseif (guardtype == ORDERGUARDTYPE)
		var myhonor := GetObjProperty(who,"honor");
		if (!myhonor)
			myhonor := {};
			myhonor[1] := yourvirtue;
			myhonor[2] := 0;
			myhonor[3] := 0;
			SetObjProperty(who,"honor", myhonor);
		endif
		var myunspent := yourvirtue - myhonor[1];
		if ( myunspent > 100 )
			myhonor[1] := myhonor[1] + (myunspent - 100);
			myhonor[2] := myhonor[2] + (myunspent - 100);
			myunspent := 100;
			SetObjProperty(who,"honor",myhonor);
			SetMyTitle(who,myhonor[2]);
		else
			SetMyTitle(who,myhonor[2]);
		endif
	endif		

	if (guardtype == ORDERGUARDTYPE)
		if ( randomint(8) < amount )
			BonusHonor (who);
		endif	
	endif
endfunction

function AwardVirtueForRes(you,them)

	if ( GetObjProperty (them, "skillloss") )
		EraseObjProperty (them, "skillloss");
		AddVirtue (you, 5);
	endif

endfunction




///////////////////
//  Figures out how much virtue the player has
///////////////////

function GetVirtueAmount (you)
	var yourvirtue := GetObjProperty (you, "virtue");
	if (!yourvirtue)
		return 0;
	else
		return yourvirtue;
	endif
endfunction

function SetMyTitle (who, virtue)

	var shrines := GetObjProperty (who, "shrines");
	if (!shrines)
		shrines := 0;
	endif
	if (shrines != 255) 
		who.title_prefix := "Squire ";
		return;
	endif

	if ( virtue < 200 )
		who.title_prefix := "Squire ";
	elseif ( virtue < 400 )
		if (who.gender)
			who.title_prefix := "Dame ";
		else
			who.title_prefix := "Sir ";
		endif
	elseif ( virtue < 800 )
		if (who.gender)
			who.title_prefix := "The Esteemed Dame ";
		else
			who.title_prefix := "The Esteemed Sir ";
		endif
	elseif ( virtue < 1600 )
		if (who.gender)
			who.title_prefix := "The Noble Dame ";
		else
			who.title_prefix := "The Noble Sir ";
		endif
	elseif ( virtue < 3200 )
		if (who.gender)
			who.title_prefix := "The Illustrious Dame ";
		else
			who.title_prefix := "The Illustrious Sir ";
		endif
	elseif ( virtue < 6400 )
		if (who.gender)
			who.title_prefix := "Lady ";
		else
			who.title_prefix := "Lord ";
		endif
	elseif ( virtue < 12800 )
		if (who.gender)
			who.title_prefix := "The Noble Lady ";
		else
			who.title_prefix := "The Noble Lord ";
		endif
	elseif ( virtue < 25600 )
		if (who.gender)
			who.title_prefix := "The Illustrious Lady ";
		else
			who.title_prefix := "The Illustrious Lord ";
		endif
	else
		if (who.gender)
			who.title_prefix := "The Glorious Lady ";
		else
			who.title_prefix := "The Glorious Lord ";
		endif
	endif

endfunction

function CanJoin(guard, recruit, guardtype, shieldtype)

	if (recruit.npctemplate) return; endif
	if (!recruit.acctname) return; endif
	if (guard == recruit) return; endif

	var yourtype := GetObjProperty(recruit,"guardtype");
	if (yourtype)
		SendSysMessage(guard,"They're already a Virtue Guard.");
		if (yourtype == guardtype)
			var newshield := CreateItemAtLocation(4550, 3687  , 0, shieldtype,1);
			if (!EquipItem(recruit,newshield))
				DestroyItem(newshield);
			endif
			return;
		endif
	endif

	var basestr := CINT (GetAttributeBaseValue (recruit, "Strength") / 10);
	if (basestr < 90)
		SendSysMessage(guard,"They don't seem strong enough, to you.");
		return;
	endif

	var leadership := GetAttribute (recruit, ATTRIBUTEID_LEADERSHIP);
	if (leadership < 90)
		SendSysMessage(guard,"They seem to lack the necessary leadership skills...");
		return;
	endif

	if ( GetVirtueAmount(recruit) < 100 )
		SendSysMessage (guard, "They don't seem to be terribly Virtuous...");
		return;
	endif

	SendSysMessage (recruit, "Do you want to join the Virtue Guards?");
	if (!YesNo (recruit, "Yes!")) 
		SendSysMessage(guard,"They don't want to join.");
		return; 
	endif

	setobjproperty(recruit,"guardtype",guardtype);
	recruit.title_prefix := "Squire ";
	recruit.title_suffix := ", Virtue Guard";
	var newshield := CreateItemAtLocation(recruit.x , recruit.y , recruit.z,
		shieldtype,1);
	EquipItem(recruit,newshield);
	PlaySoundEffect(recruit, SFX_SPELL_BLESS);
	PlayObjectCenteredEffect( recruit, FX_BLESS_EFFECT, 10,10);

	SendSysMessage(guard,"Tell them to MAKE US PROUD!");

endfunction

function BestowHonor (me, you, howmuch := 0)
	if (you.npctemplate)
		return;
	elseif (!you.acctname)
		return;
	elseif (me == you)
		return;
	endif

	var yourtype := GetObjProperty(you,"guardtype");
	if (!yourtype)
		SendSysMessage (me, "That's not a Virtue Guard");
		return; 
	endif
	if (yourtype != ORDERGUARDTYPE)
		SendSysMessage (me, "That's not an Order Guard");
		return;
	endif

//honor[1] is how many points I've spent, honor[2] is points received
//honor[3] is the last person you honored, so it takes 
//at least three twinks to sploit

	var myvirtue := GetVirtueAmount (me);
	var myhonor := GetObjProperty(me,"honor");
	if (!myhonor)
		myhonor := {};
		myhonor[1] := myvirtue;
		myhonor[2] := 0;
		myhonor[3] := 0;
	endif

	var yourvirtue := GetVirtueAmount (you);
	var yourhonor := GetObjProperty (you, "honor");
	if (!yourhonor)
		yourhonor := {};
		yourhonor[1] := yourvirtue;
		yourhonor[2] := 0;
		yourhonor[3] := 0;
	endif

	if (myhonor[3] != you.serial)
		var myunspent := myvirtue - myhonor[1];
		if (howmuch)
			myunspent := cint(myunspent/2);
		endif
		if ( myunspent > 0)
			yourhonor[2] := yourhonor[2] + myunspent;
			myhonor[1] := myhonor[1] + myunspent;
			myhonor[3] := you.serial;
			SetMyTitle(you,yourhonor[2]);
			SetObjProperty(me,"honor",myhonor);
			SetObjProperty(you,"honor",yourhonor);
		endif
	endif

	if (!howmuch)
		PerformAction(me,ANIM_BOW);
		SendSysMessage(me,"You honor " + you.name);
		SendSysMessage(you,me.name + " honors you");
	else
		PerformAction (me, ANIM_SALUTE);
		SendSysMessage (me, "You respect " + you.name);
		SendSysMessage (you, me.name + " respects you");
	endif

endfunction

function BonusHonor (me, amt := 1)
	//we have to make up for "spent points" leaving the system via
	//capped characters, quitting character, and recriminations:

	var myhonor := GetObjProperty (me, "honor");
	if (!myhonor)
		return;
	endif
	myhonor[1] := myhonor[1] - amt;
	SetObjProperty (me, "honor", myhonor);
endfunction

function Censure( me, you )

	if (you.npctemplate) return; endif
	if (!you.acctname) return; endif
	if (me == you) return; endif

	var yourtype := GetObjProperty(you,"guardtype");
	if (!yourtype)
		SendSysMessage(me,"That's not a Virtue Guard");
		return; 
	endif
	if (yourtype != ORDERGUARDTYPE)
		SendSysMessage(me,"That's not an Order Guard");
		return;
	endif

//honor[1] is how many points I've spent, honor[2] is points received
//honor[3] is the last person you honored, so it takes 
//at least three twinks to sploit

	var myvirtue := GetVirtueAmount (me);
	var myhonor := GetObjProperty(me,"honor");
	if (!myhonor)
		myhonor := {};
		myhonor[1] := 0;
		myhonor[2] := 0;
		myhonor[3] := 0;
	endif

//	var yourvirtue := GetVirtueAmount (you);
	var yourhonor := GetObjProperty(you,"honor");
	if (!yourhonor)
		yourhonor := {};
		yourhonor[1] := 0;
		yourhonor[2] := 0;
		yourhonor[3] := 0;
	endif

	if (myhonor[3] != you.serial)
		var myunspent := myvirtue - myhonor[1];
		myunspent := cint(myunspent/10);
		if ( myunspent > 0)
			yourhonor[2] := yourhonor[2] - myunspent;
			myhonor[1] := myhonor[1] - myunspent;
			myhonor[3] := you.serial;
			SetMyTitle(you,yourhonor[2]);
			SetMyTitle(me,myhonor[2]);
			SetObjProperty(me,"honor",myhonor);
			SetObjProperty(you,"honor",yourhonor);
		endif
	endif

	PerformAction(me,ANIM_FIDGET_2);
	sleepms(500);
	PerformAction(me,ANIM_FIDGET_1);
	sleepms(500);
	PerformAction(me,ANIM_EAT);
	SendSysMessage(me,"You censure " + you.name);
	SendSysMessage(you,me.name + " has censured you");

endfunction

function SetNonVGTitle( who )

	if ( GetObjProperty(who,"guardtype") )
		return;
	endif

	var virtue := GetVirtueAmount (who);
	if (!virtue) return; endif

	if ( virtue < -25600 )
		who.title_prefix := "The Sinister ";
	elseif ( virtue < -12800 )
		who.title_prefix := "The Vile ";
	elseif ( virtue < -6400 )
		who.title_prefix := "The Ignoble ";
	elseif ( virtue < -3200 )
		who.title_prefix := "The Malicious ";
	elseif ( virtue < -1600 )
		who.title_prefix := "The Dastardly ";
	elseif ( virtue < -800 )
		who.title_prefix := "The Dishonorable ";
	elseif ( virtue < -400 )
		who.title_prefix := "The Unsavory ";
	elseif ( virtue < -200 )
		who.title_prefix := "The Rude ";
	elseif ( virtue < 200 )
		who.title_prefix := "";
	elseif ( virtue < 400 )
		who.title_prefix := "The Honorable ";
	elseif ( virtue < 800 )
		who.title_prefix := "The Commendable ";
	elseif ( virtue < 1600 )
		who.title_prefix := "The Admirable ";
	elseif ( virtue < 3200 )
		who.title_prefix := "The Reknowned  ";
	elseif ( virtue < 6400 )
		who.title_prefix := "The Famed ";
	elseif ( virtue < 12800 )
		who.title_prefix := "The Noble ";
	elseif ( virtue < 25600 )
		who.title_prefix := "The Illustrious ";
	else
		who.title_prefix := "The Glorious ";
	endif

endfunction
