use uo;
use os;
use util;
use cfgfile;

include "include/client";
include "include/objtype";
include "../pkg/tools/random/random";
include "include/myutil";

var npc_cfg_file := ReadConfigFile (":*:npc_lootgroups");
var chest_cfg_file := ReadConfigFile (":*:chest_lootgroups");
var itemgroup_cfg_file := ReadConfigFile (":loot:itemgroups");

///////////////////
//  used to make loot on corpses.  Probably only called by death.src.  Lootgroups for NPCs
//  can be found in npc_lootgroups.cfg
///////////////////

function MakeLootOnCorpse (corpse, npctemplate)
	var elem := FindConfigElem (npc_cfg_file, npctemplate);
	if (!elem)
		return 0;
	endif

	if (elem.gold)
		var gold := RandomDiceRoll (elem.gold);
		CreateItemInContainer (corpse, UOBJ_GOLD_COIN, gold);
	endif

	if (elem.chest)
		MakeLockedChest (corpse, elem.chest, elem.chest_quality);
	endif

	// Ackadian Change to add glowies
	if (elem.tool_chance)
		MakeMagicCraftingTool (corpse, elem.tool_chance);
	endif

	foreach str in GetConfigStringArray (elem, "item")
		MakeSpecifiedLootItem (corpse, str);
	endforeach

	foreach str in GetConfigStringArray (elem, "random")
		MakeItemsOutOfGroup (corpse, str);
	endforeach

	var quality := elem.magic_quality;
	if (quality)
		foreach str in GetConfigStringArray (elem, "magic_group")
			var temp_str := SplitWords (str);
			var group := temp_str [1];
			var chance := CInt (temp_str [2]);
			if (RandomInt (4500) + 1 <= chance)
				MakeMagicItemOutOfGroup (corpse, group, quality);
			endif
		endforeach
	endif
	return 1;

endfunction




///////////////////
//  used to make loot in chests.  Lootgroups can be found in chest_lootgroups.cfg
///////////////////

function MakeLootInChest (chest, lootgroup)

	var elem := FindConfigElem (chest_cfg_file, lootgroup);
	if (!elem)
		return 0;
	endif

	if (elem.gold)
		var gold := RandomDiceRoll (elem.gold);
		CreateItemInContainer (chest, UOBJ_GOLD_COIN, gold);
	endif

	foreach str in GetConfigStringArray (elem, "item")
		MakeSpecifiedLootItem (chest, str);
	endforeach

	foreach str in GetConfigStringArray (elem, "random")
		MakeItemsOutOfGroup (chest, str);
	endforeach

	var quality := elem.magic_quality;
	if (quality)
		foreach str in GetConfigStringArray (elem, "magic_group")
			var temp_str := SplitWords (str);
			var group := temp_str [1];
			var chance := CInt (temp_str [2]);
			if (RandomInt (3000) + 1 <= chance)
				MakeMagicItemOutOfGroup (chest, group, quality);
			endif
		endforeach
	endif
	//Ackadian Change to add traps to loot chests
	if (RandomInt (3) == 1)
		case (RandomInt (3))
			0:
				SetObjProperty (chest, "trap", "explosion");
			1:
				SetObjProperty (chest, "trap", "poison");
			2:
				SetObjProperty (chest, "trap", "djinni");
		endcase
	endif
	return 1;

endfunction




///////////////////
//  creates specific item in container
///////////////////

function MakeSpecifiedLootItem (container, str)
	str := SplitWords (str);
	var itemname := str[1];
	var number := RandomDiceRoll (str[2]);
	var chance := CInt(str[3]);

	if (number > 20)
		number := 20;
	endif

	for i := 1 to number
		if (RandomInt(100) + 1 <= chance)
			var item := CreateItemInContainer (container, itemname, 1);
			item.movable := 1;
		endif
	endfor
	return 1;

endfunction




///////////////////
//  creates random items out of a group of possible items
///////////////////

function MakeItemsOutOfGroup (container, str)
	str := SplitWords (str);
	var groupname := str[1];
	var number := RandomDiceRoll (str[2]);
	var chance := CInt(str[3]);

	if (number > 100)
		number := 100;
	endif

	var elem := FindConfigElem (itemgroup_cfg_file, groupname);
	if (!elem)
		return 0;
	endif

	var possible_items_array := GetConfigStringArray (elem, "item");
	var total_items := len (possible_items_array);
	var item, itemtype, itemnumber;
	for i := 1 to number
		if (RandomInt (150) + 1 <= chance)  // bumped up for raw
			itemnumber := RandomInt (total_items) + 1;
			itemtype := possible_items_array [itemnumber];
			if (CINT (itemtype))
				itemtype := CINT (itemtype);
			endif
			item := CreateItemInContainer (container, itemtype, 1);
			item.movable := 1;
		endif
	endfor
	return item;

endfunction




///////////////////
//  makes a magic item out of the group specified
///////////////////

function MakeMagicItemOutOfGroup (container, str, quality)

	var elem := FindConfigElem (itemgroup_cfg_file, str);
	if (!elem)
		return 0;
	endif

	var possible_items_array := GetConfigStringArray (elem, "item");
	var total_items := len (possible_items_array);
	var itemnumber := RandomInt (total_items) + 1;
	var itemtype := possible_items_array [itemnumber];

	var item := CreateItemInContainer (container, itemtype, 1);
	item.movable := 1;

	var parms := {};
	parms[1] := item;
	parms[2] := CInt (quality);

	run_script_to_completion (":loot:makeitemmagic", parms);
	return item;

endfunction




///////////////////
//  makes a locked chest that, when picked, spawns loot
///////////////////

function MakeLockedChest (container, chest_chance, chest_quality)

	if (chest_chance < RandomInt (100))
		return 0;
	endif

	var base_quality := CINT(chest_quality/2) + RandomInt(CINT(chest_quality/2));
	var random_quality := CINT (((RandomInt (10)+1) * (RandomInt (10)+1))/10) + 1;
	if (random_quality > base_quality)
		base_quality := random_quality;
	endif

	if (base_quality > 10)
		base_quality := 10;
	endif

	var chest := CreateItemInContainer (container, 0x0E79, 1);
	if (!chest)
		return 0;
	endif

	SetObjProperty (chest, "mobchest_spawntime", ReadGameClock ());
	SetObjProperty (chest, "lockpicking", base_quality);
	SetObjProperty (chest, "questitem", 1);
	chest.locked := 1;
	chest.movable := 1;
	case (chest_quality)
		10:
		9:
			chest.color := 1846;
		8:
		7:
			chest.color := 1848;
		6:
		5:
			chest.color := 1849;
		4:
		3:
			chest.color := 1851;
		default:
			chest.color := 1853;
	endcase	
	chest.name := "a locked chest";

	//make traps
	if (RandomInt (3) == 1)
		case (RandomInt (3))
			0:
				SetObjProperty (chest, "trap", "explosion");
			1:
				SetObjProperty (chest, "trap", "poison");
			2:
				SetObjProperty (chest, "trap", "djinni");
		endcase
	endif

	return 1;

endfunction

///////////////////
//  Creates a magic crafting tool
///////////////////

function MakeMagicCraftingTool (corpse, tool_chance)
		var chance := random (375);
		if (tool_chance < chance )
			
			return 0;
		endif
		

	

	var objtype, objname, usescript;
	case (RandomInt(6))
		0:	//jointingplane
			objtype := 0x1030;
			objname := "jointing plane";
			usescript := ":carpentry:enchantment";
		1:	//sewingkit
			objtype := 0xF9D;
			objname := "sewing kit";
			usescript := ":tailoring:enchantment";
		2:	//tongs
			objtype := 0xFBB;
			objname := "pair of tongs";
			usescript := ":blacksmithy:enchantment";
		3:	//smithyhammer
			objtype := 0x13e3;
			objname := "smithing hammer";
			usescript := ":blacksmithy:enchantment";
		4:	//tinkertools
			objtype := 0x1ebc;
			objname := "tinker tools";
			usescript := ":tinkering:enchantment";
		5:	//scroll
			objtype := 0x1ae4;
			objname := "skull";
			usescript := ":necromancy:enchantment";
	
	endcase

	var it := CreateItemInContainer (corpse, objtype, 1);
	if (!it)
		return;
	endif

	it.name := "a glowing " + objname;
	it.color := 1154;
	setobjproperty (it, "power", (RandomInt (tool_chance)+1)+ RandomInt (5));
	it.usescript := usescript;
	it.buyprice := -1;



endfunction


