include "include/dotempmods";

///////////////////
//  Casts the specified spell
///////////////////

function castspell ( byref caster, byref cast_on, spellname )
	var parms := array;
	parms [1] := caster;
	parms [2] := cast_on;
	parms [3] := spellname;
	run_script_to_completion (":npcs:npc_spellcast", parms);
endfunction





///////////////////
//  picks an attack spell to cast out of those known by the NPC, chosen out of those listed in NPCDesc
///////////////////

function cast_offensive_spell (byref caster, byref cast_on)
	var iamgood := 0;
	if (IsGood (caster))
		iamgood := 1;
	endif

	//if we're fighting a summoned creature, try to dispel them and attack their master
	if (GetObjProperty (cast_on, "summoned") )
		CastSpell (caster, cast_on, ":summoning_spells:dispel");
		if (GetObjProperty (cast_on, "master"))
			var nexttarget := SystemFindObjectBySerial (GetObjProperty (cast_on, "master"));
			if ( nexttarget )
				sleep(1);
				if (!cast_on)
					Fight (nexttarget);
					return;
				else
					return;
				endif
			endif
		endif
	endif

	//now we pick a random spell out of the spells we know, and do some checking to make sure its
	//a resonably intelligent choice
	var MyTemplateElem := GetNpcTemplateElem (me.npctemplate);
	var spells := GetConfigStringArray (MyTemplateElem, "spell");
	if (!spells)
		return;
	endif

	var thespell;
	var castit := 0;
	var tries := 0;
	while (!castit and tries < 20)
		thespell := spells[RandomInt (len(spells))+1];
		tries := tries + 1;
		castit := 1;
		if ( cint(GetObjProperty( cast_on, "mr")) )
			if (randomint(2))
				thespell := ":damage_spells:magicarrow";
			endif
		elseif ( thespell == ":statmod_spells:curse" )
			if (!CanMod(cast_on,"call"))
				castit := 0;
			endif
		elseif ( thespell == ":statmod_spells:weaken" )
			if (!CanMod(cast_on,"cstr"))
				castit := 0;
			endif
		elseif ( thespell == ":statmod_spells:clumsy" )
			if (!CanMod(cast_on,"cdex"))
				castit := 0;
			endif
		elseif ( thespell == ":statmod_spells:feeblemind" )
			if (!CanMod(cast_on,"cint"))
				castit := 0;
			endif
		elseif ( thespell == ":statmod_spells:polymorph" )
			if (!CanMod(cast_on,"cpoly"))
				castit := 0;
			endif
		elseif (thespell == ":damage_spells:explosion" or thespell == ":damage_spells:meteor_swarm")
			var good_npcs := CountNPCsInAreaOfType (cast_on.x, cast_on.y, cast_on.z, 3, "good");
			var evil_npcs := CountNPCsInAreaOfType (cast_on.x, cast_on.y, cast_on.z, 3, "evil");
			if (iamgood)
				if (good_npcs > evil_npcs)
					castit := 0;
				endif
			else
				if (evil_npcs > good_npcs)
					castit := 0;
				endif
			endif
		elseif (thespell = ":damage_spells:chain_lightning" or thespell = ":damage_spells:earthquake")
			var range := GetSkill (caster,SKILLID_MAGERY)/15;
			var good_npcs := CountNPCsInAreaOfType (cast_on.x, cast_on.y, cast_on.z, range, "good");
			var evil_npcs := CountNPCsInAreaOfType (cast_on.x, cast_on.y, cast_on.z, range, "evil");
			if (iamgood)
				if (good_npcs > evil_npcs)
					castit := 0;
				endif
			else
				if (evil_npcs > good_npcs)
					castit := 0;
				endif
			endif
		elseif (thespell["field"])
			if (cast_on.x == me.x or cast_on.y == me.y)
				if (Distance (me, cast_on) < 4)
					castit := 0;
				endif
			endif
		elseif ( ( thespell == ":misc_spells:manadrain" or thespell == ":misc_spells:manavamp" ) and (cast_on.mana < 10) )
			castit := 0;
		endif
	endwhile

	CastSpell (caster, cast_on, thespell);
endfunction




function CountNPCsInAreaOfType (x, y, z, range, type)
	var counter := 0;
	foreach mobile in ListMobilesNearLocationEx (x, y, z, range, 0x3)
		if (type == "good")
			if (IsGood (mobile) )
				counter := counter + 1;
			endif
		elseif (type == "evil")
			if (IsHostile (mobile) )
				counter := counter + 1;
			endif
		endif
	endforeach
	return counter;
endfunction




///////////////////
//  determines the spellID for the given spell
///////////////////

Function GetSpellID (spellname)

	var spellid := 0;

	case (spellname)
	":statmod_spells:clumsy":			spellid := 1;
	":statmod_spells:feeblemind":			spellid := 3;
	":misc_spells:heal":				spellid := 4;
	":damage_spells:magicarrow":			spellid := 5;
	":misc_spells:nightsight":			spellid := 6;
	":statmod_spells:reactivearmor":		spellid := 7;
	":statmod_spells:weaken":			spellid := 8;
	":statmod_spells:agility":			spellid := 9;
	":statmod_spells:cunning":			spellid := 10;
	":poison_spells:cure":				spellid := 11;
	":damage_spells:harm":				spellid := 12;
	":statmod_spells:protection":			spellid := 15;
	":statmod_spells:strength":			spellid := 16;
	":statmod_spells:bless":				spellid := 17;
	":damage_spells:fireball":			spellid := 18;
	":poison_spells:poison":				spellid := 20;
	":travel_spells:teleport":			spellid := 22;
	":field_spells:wallofstone":			spellid := 24;
	":poison_spells:archcure":			spellid := 25;
	":statmod_spells:archprot":			spellid := 26;
	":statmod_spells:curse":				spellid := 27;
	":field_spells:firefield":			spellid := 28;
	":misc_spells:gheal":				spellid := 29;
	":damage_spells:lightning":			spellid := 30;
	":misc_spells:manadrain":			spellid := 31;
	":summoning_spells:blade_spirit":		spellid := 33;
	":field_spells:dispel_field":			spellid := 34;
	":misc_spells:reflect":				spellid := 36;
	":damage_spells:mindblast":			spellid := 37;
	":statmod_spells:paralyze":			spellid := 38;
	":field_spells:poisonfield":			spellid := 39;
	":summoning_spells:summon_creature":	spellid := 40;
	":summoning_spells:dispel":			spellid := 41;
	":damage_spells:ebolt":				spellid := 42;
	":damage_spells:explosion":			spellid := 43;
	":misc_spells:invisibility":			spellid := 44;
	":statmod_spells:masscurse":			spellid := 46;
	":field_spells:parafield":			spellid := 47;
	":misc_spells:reveal":				spellid := 48;
	":damage_spells:chain_lightning":		spellid := 49;
	":field_spells:energy_field":			spellid := 50;
	":damage_spells:fstrike":			spellid := 51;
	":misc_spells:manavamp":				spellid := 53;
	":summoning_spells:massdispel":		spellid := 54;
	":damage_spells:meteor_swarm":		spellid := 55;
	":statmod_spells:polymorph":			spellid := 56;
	":damage_spells:earthquake":			spellid := 57;
	":summoning_spells:vortex":			spellid := 58;
	":misc_spells:resurrect":			spellid := 59;
	":summoning_spells:summon_air":		spellid := 60;
	":summoning_spells:summon_daemon":		spellid := 61;
	":summoning_spells:summon_earth":		spellid := 62;
	":summoning_spells:summon_fire":		spellid := 63;
	":summoning_spells:summon_water":		spellid := 64;
	":necro_spells:frostfield":			spellid := 77;
	":necro_spells:darkness":			spellid := 78;
	":druid_spells:graspingroots":		spellid := 81;
	":druid_spells:stingingswarm":		spellid := 82;
	":druid_spells:tidalwave":			spellid := 83;
	":druid_spells:natures_curse":		spellid := 92;
	endcase

	return spellid;
endfunction










