/*

There's a "hit list" variable used by the system to make sure everybody
gets virtue (and, thus, warrior experience).  The problem is that it
has "global variable itis" in that every script on the earth seems to
muck with it directly.  I have no idea what changes to the hit list
would break various things so i've opted not to touch it.

These functions use the following object properties: 
#apcredit	// the list of people who get credit
#aplastcredit	// the last time the credit was modified
#aplasthit	// serial of the last creature to hurt me

#aplasthelper	// the last person to help me (heal, etc)
#aphelptime	// the last time i was helped by said person

Touch these through anything but the below functions and I will smite you.

These routines are intended to be more generous than the tactics virtue
awarding routines, as we have to award multiple playstyles so players
with more eclectic abilities than hitting things in the head can advance.

*/

function perknotehelp(helper,helpee)
//	paranoia
	if (!helper || !helpee)
		return 0;
	endif

	SetObjProperty(helpee,"#aphelptime",ReadGameClock());

//	let's recurse up to the master
	while (helper.master)
		helper := helper.master;
	endwhile

//	let's recurse up to the master
	while (helpee.master)
		helpee := helpee.master;
	endwhile
	
	if (helper.acctname)
		SetObjProperty(helpee,"#aplasthelper",helper.serial);
	endif
	
	return 1;
endfunction

function perktap(me,monster)
//	paranoia
	var dirty := 0;

	if (!me || !monster)
		return 0;
	endif
	
//	make sure monster is a monster
	if (!(IsKillableNPC(monster)))
		return 0;
	endif

//	if a monster hasn't been hit in 5 minutes, wipe the list
//	and start anew.  This essentially "ends fights".
	var lc := GetOPI(monster,"#aplastcredit",ReadGameClock());
	if ( lc+300<ReadGameClock() )
		SetObjProperty(monster,"#apcredit",{});
	endif
	
	SetObjProperty(monster,"#aplastcredit",ReadGameClock());
	
//	let's recurse up to the master
	while (me.master)
		me := me.master;
	endwhile

	var c := GetOPI(monster,"#apcredit",{});


//	make the monster angry at me
	var s := me.serial;
	
	if (s in c)
//		sendsysmessage(me,"" + monster.name + " hates " + me.name + " already!");	
	else
//		sendsysmessage(me,"" + monster.name + " hates " + me.name + "!");
		c.append(s);
		
		dirty := 1;

	endif
	
	
//	make the monster angry at the last person who helped me

//	only give credit to people who help if they've helped within
//	five minutes.
	var lh := GetOPI(me,"#aphelptime",ReadGameClock());
	var h := GetOPI(me,"#aplasthelper",0);
	
//	sendsysmessage(me,"lh : " + lh + " h: " + h + " RGC(): " + ReadGameClock());
	
	if ( lh+300>ReadGameClock() && h)	
		if (h in c)
//			sendsysmessage(me,"" + monster.name + " hates " + me.name + " already!");	
		else
//			sendsysmessage(me,"" + monster.name + " hates " + me.name + "!");
			c.append(h);
		
			dirty := 1;

		endif
	endif


//	if necessary, write #apcredit
	if (dirty)	
		SetObjProperty(monster,"#apcredit",c);	
	endif
endfunction

/*
snitched from nuke, slight alterations
*/
function IsKillableNPC (mobile)
	if (!mobile.npctemplate)
		return 0;
	elseif (GetObjProperty (mobile, "master") )
		return 0;
	elseif (GetObjProperty (mobile, "myhome") )
		return 0;
	elseif (GetObjProperty (mobile, "summoned") )
		return 0;
	endif
	return 1;
endfunction

/*
snitched from AwardVirtue (in death.src), many alterations
*/
function perkgivecredit(corpse, npctemplate)
//	printtextabove(corpse,"*dies*");

	var lc := GetOPI(corpse,"#aplastcredit",ReadGameClock());
	if ( lc+300<ReadGameClock() )
		SetObjProperty(corpse,"#apcredit",{});
	endif

	var hitlist := GetOPI (corpse, "#apcredit",{});
	if (len(hitlist)<1)
		return;
	endif
	
//	printtextabove(corpse,"*" + hitlist + "*");

	var npccfg := ReadConfigFile (":*:npcdesc");
	var virtue := cint(npccfg[npctemplate].virtue);
	var hostile := cint(npccfg[npctemplate].hostile);
	
	if ( (!virtue) || (virtue < 0) )
		virtue := 0;
	endif
	
	if (npccfg[npctemplate].hostile)
		virtue := virtue + 1;
	endif

	if (npccfg[npctemplate].alignment == "evil")
		virtue := virtue + 1;
	endif
	
	var isboss := getobjproperty( corpse, "bosstype" );
	if (isboss)
		virtue := virtue + 1;
	endif
	
//	printtextabove(corpse,"-=" + virtue + "=-");
	
	foreach person in hitlist
		var you := SystemFindObjectBySerial (person);
		if (!you)
			you := SystemFindObjectBySerial(person,SYSFIND_SEARCH_OFFLINE_MOBILES);
		endif
		
		if (you)
			if (virtue > 0)
				perkaddxp(you,virtue);
			endif			
		endif
	endforeach

	EraseObjProperty (corpse, "#aplastcredit");
	EraseObjProperty (corpse, "#apcredit");

endfunction