include "../pkg/character/skillcheck/skillcheck";

function findinstrument (character)
	if (GetObjProperty (character, "instrument"))
		var instrument_serial := GetObjProperty (character, "instrument");
		foreach item in EnumerateItemsInContainer (character.backpack)
			if (item.serial == instrument_serial)
				if (ReserveItem (item) or GetObjProperty (character, "singing"))
					return item;
				endif
			endif
		endforeach
	endif

	foreach item in EnumerateItemsInContainer (character.backpack)
		case (item.objtype)
			0x7067:
			0x0e9d:
			0x0e9e:
			0x0eb3:
			0x0eb4:
			0x0e9c:
			0x0eb2:
			0x0eb1:
				if (ReserveItem (item))
					SetObjProperty (character, "instrument", item.serial);
					return item;
				endif
		endcase
	endforeach

	return 0;
endfunction

function can_sing (me, circle)
	var cfg := ReadConfigFile (":*:itemdesc");
	foreach item in ListEquippedItems (me)
		if (cfg[item.objtype].blockcircle && cfg[item.objtype].blockcircle <= circle) 
			SendSysmessage(me, "Your equipment prevents you from using this instrument!");
			return 0;
		endif
	endforeach

	return 1;
endfunction

function PlayNoise (me, instrument, difficulty)
	var cfg := ReadConfigFile (":musicianship:itemdesc");
	var elem := FindConfigElem (cfg, instrument.objtype);

	if (!difficulty)
		MakeSound (me, elem.succeed);
		return 1;
	elseif (CheckSkillWithItem (me, instrument, SKILLID_MUSICIANSHIP, difficulty))
		MakeSound (me, elem.succeed);
		return 1;
	else
		MakeSound (me, elem.fail);
		return 0;
	endif
endfunction




///////////////////
//  does the sound effect for the musical instrument, dependant on if people want to hear it or not
///////////////////

function MakeSound (character, soundeffect)
	foreach mobile in ListMobilesNearLocationEx (character.x, character.y, character.z, 15, 
			LISTEX_FLAG_NORMAL+LISTEX_FLAG_HIDDEN+LISTEX_FLAG_GHOST)
		if (mobile.acct)
			var acct := mobile.acct;
			var sound_options := acct.GetProp ("sound_options");
			if (!sound_options or !sound_options[1])
				PlaySoundEffectPrivate (character, soundeffect, mobile);
			endif
		endif
	endforeach
endfunction

/////////////////
///  From this point down was added for songbooks
/////////////////

function PracticeSongInfo(songnumber)
	var config := ReadConfigFile("practicesongs");
	return config[songnumber];
endfunction

function SongInfo(songnumber)
	var config := ReadConfigFile("bardsongs");
	return config[songnumber];
endfunction

function ReleaseScript(pid)
	var process := GetProcess(pid);
	process.SendEvent("done");
endfunction

function GetHarmonizedBards(who, range:=5)
	var numbards := 0;
	foreach person in ListMobilesInLineOfSight(who, range)
		if ( GetObjProperty(person, "#harmonizing") == who.serial )
			numbards := numbards + 1;
		endif
	endforeach
	
	return numbards;
endfunction

function BardTarget(byref who, byref npctarget, prompt := "Select a target", ops := TGTOPT_CHECK_LOS + TGTOPT_NEUTRAL)
	var x := who.x;
	var y := who.y;
	who.hidden := 0;
	if (npctarget)
		return npctarget;
	endif
	
	if (who.npctemplate)
		return;
	endif
	
	SendSysMessage(who, prompt);
	
	var targ := target(who, ops);

	if (targ.hidden or targ.concealed) 
    		SendSysMessage (who, "Invalid target"); 
    		return 0; 
	endif
	
	if ( ( who.x == x ) && ( who.y == y ) )
		return targ;
	else
		SendSysMessage(who, "You stop playing your song.");
		return 0;
	endif
endfunction

function BardTargetCoordinates(byref who, byref params := "", prompt := "Select a target")
	var x := who.x;
	var y := who.y;
	
	who.hidden := 0;
	if (params[3])
		return params[3];
	elseif ( who.npctemplate )
		return;
	endif
	
	SendSysMessage(who, prompt);
	var targ := TargetCoordinates(who);
	
	if ( ( who.x == x ) && ( who.y == y ) )
		return targ;
	else
		SendSysMessage(who, "You stop playing your song.");
		return 0;
	endif
endfunction

function IsInGroup(bard, mobile, chklos := 0)
	if (bard == mobile)
		return 1;
	endif
	
	if ( (chklos) && ! (CheckLineOfSight(bard, mobile)) )
		return 0;
	elseif ( !( mobile.serial in GetObjProperty(bard, "BardGroup") ) )
		return 0;
	else
		return 1;
	endif
endfunction

function CheckStam(who, amount)
	if ( GetVital(who,"Stamina") >= amount )
		SetVital(who,"Stamina",GetVital(who,"Stamina") - amount); 
		return 1;
	else
		return 0;
	endif
endfunction



