//  This file contains the functions that should end up being common between
//  armorsmithing, blacksmithing, tailoring, and possibly some of tinkering
//  excluding the gump work which is in craftinggump.inc


//  Makes the baic quality version of an item
function makeitem(character, SKILLID, item, cfgfile, makematerial, upgradematerial, targetquality, qty, sound, requiresanvil := 0)
	var iteminfo := FindConfigElem (cfgfile, item);
	
	var characterx := character.x;
	var charactery := character.y;
	var craftingbag := FindCraftingBag (character);
	var items_to_make := qty;
	
	var materialinfo := splitcomponent(iteminfo.material);
	
	var sounddelay := 0;
	if (materialinfo[1] == "cloth" or materialinfo[1] == "leather")
		sounddelay := 5;
	else
		sounddelay := 2;
	endif
	
	if (requiresanvil)
		if (!SearchForAnvilInArea (character))
			SendSysmessage (character, "You must be near an anvil to make anything.");
			return;
		endif
		
		if (!FindForgeInArea (character))
			SendSysmessage (character, "You must be near a forge to make anything.");
			return;
		endif
		
		if (!CheckSkillForDifferentMetals (character, makematerial.objtype, SKILLID))
			sendsysmessage(character, "our skill is too low to work with that material!");
			return;
		endif
	endif
		
	if (!Accessible (character, makematerial))
		SendSysMessage (character, "You can't reach the material to make the item.");
		return;
	endif
		
	if (makematerial.amount < materialinfo[2])
		sendsysmessage(character, "You do not have enough material to start the process.");
		return;
	endif

	var created_item;
	repeat
		created_item := 0;
		
		if (!ReserveItem(makematerial))
			SendSysmessage (character, "You cannot use that material right now.");
			return;
		endif

		for i := 1 to iteminfo.time
			i := i;  // just to make the warning go away
			PlaySoundEffect (character, sound);
			sleep (sounddelay);
		endfor
		PlaySoundEffect (character, sound);
		
		if (!CheckSkill (character, GetSkillIDByAttributeID (SKILLID), iteminfo.skill, 0 ) )
			var lossamount := RandomInt (CINT ( materialinfo[2]/3) )+1;
			SubtractAmount (makematerial, lossamount);
			SendSysmessage (character, "You fail, destroying some " + materialinfo[1] + "!");
		else
			created_item := CreateItemInContainer (craftingbag, item, 1);
			if (!created_item)
				PrintTextAbovePrivate (character, "*Your backpack is full!*", character);
				releaseitem(upgradematerial);
				return;
			endif

			SendSysMessage (character, "You place the item in your backpack.");
			SubtractAmount (makematerial, materialinfo[2]);
			created_item.name := "a " + iteminfo.name + " [crafted by " + character.name + "]";
			releaseitem(makematerial);
			if (iteminfo.hasquality)
				created_item.quality := 1; 
				created_item.hp := created_item.maxhp;
				if (makematerial.color)
					created_item.color := makematerial.color;
				endif
				if (!upgradematerial)
					sendsysmessage(character, "You have not selected " + materialinfo[1] + " to use for upgrading.");
					return;
				else
					if (!upgradeitemquality(character, created_item, iteminfo, upgradematerial, SKILLID, targetquality, sound, requiresanvil))
						if (upgradematerial.amount < materialinfo[2]/2)
							sendsysmessage(character, "You no longer have enough " + materialinfo[1] + " to continue attempt to upgrade.");
							return;
						endif
					endif
				endif
			endif
			created_item.buyprice := -1;
			
		endif
		
		if (characterx != character.x or charactery != character.y)
			SendSysMessage (character, "You stop smithing.");
			return;
		endif
		
		if (created_item)
			items_to_make := items_to_make - 1;
		endif
		sleep (2);
	until (!items_to_make);
endfunction



// Upgrades an existing Item's quality level to the level set in the gump
// returns 0 on failure 1 on success
function upgradeitemquality(character, upgradeitem, iteminfo, upgradematerial, SKILLID, targetquality, sound, requiresanvil :=0)
	var characterx := character.x;
	var charactery := character.y;
	
	var materialinfo := splitcomponent(iteminfo.material);
	
	var sounddelay := 0;
	if (materialinfo[1] == "cloth" or materialinfo[1] == "leather")
		sounddelay := 5;
	else
		sounddelay := 2;
	endif
	
	if (requiresanvil)
		if (!SearchForAnvilInArea (character))
			SendSysmessage (character, "You must be near an anvil to make anything.");
			return;
		endif
			
		if (!FindForgeInArea (character))
			SendSysmessage (character, "You must be near a forge to make anything.");
			return;
		endif
			
		if (!CheckSkillForDifferentMetals (character, upgradematerial.objtype, SKILLID))
			sendsysmessage(character, "our skill is too low to work with that material!");
			return;
		endif
	endif
	
	if(upgradeitem.name["xceptional"])
		sendsysmessage(character, "That item is already of exeptional quality!");
		return;
	endif
	
	if (!upgradeitem.name["crafted by"])
		sendsysmessage(character, "You can only upgrade player crafted items.");
		return;
	endif
	
	if (!Accessible (character, upgradematerial))
		SendSysMessage (character, "You can't reach the " + materialinfo[1] + " to make the item.");
		return;
	endif
	
	if (!ReserveItem(upgradematerial))
		SendSysmessage (character, "You cannot use that " + materialinfo[1] + " right now.");
		return;
	endif
		
	while (upgradeitem.quality < targetquality and characterx == character.x and charactery == character.y)
		var materialneeded := CINT (materialinfo[2]/2);
		
		if (!upgradematerial or upgradematerial.amount < materialneeded)
			SendSysMessage (character, "You don't have enough " + materialinfo[1] + " to upgrade that item!");
			return;
		endif
		
		var upgrade_skill := iteminfo.skill;
		if (upgradeitem.desc["quality"])
			upgrade_skill := upgrade_skill + 25;
		else
			upgrade_skill := upgrade_skill + 15;
		endif
		
		if (upgrade_skill > 110)
			upgrade_skill := 110;
		endif
		
		if (upgrade_skill > GetAttribute (character, SKILLID) + 20)
		SendSysMessage (character, "Your skill is too low to upgrade that item.");
			return 0;
		endif
				
		var upgrade_timedelay := CINT (iteminfo.time) - 1;
		if (upgrade_timedelay < 1)
			upgrade_timedelay := 1;
		endif	
		
		for i := 1 to upgrade_timedelay
			i := i; // just to make the warning go away
			PlaySoundEffect(character, sound);
			sleep (sounddelay);
		endfor
		PlaySoundEffect(character, sound);
		if (CheckSkill (character, GetSkillIDByAttributeID (SKILLID), upgrade_skill, 0 ) )
			SendSysMessage (character, "You succeed in upgrading the item.");
			SubtractAmount (upgradematerial, materialneeded);
			if (upgradeitem.desc ["quality"])
				upgradeitem.name := "an exceptional " + TruncateArticle(iteminfo.name) + " [crafted by " + character.name + "]";
			else
				upgradeitem.name := "a quality " + TruncateArticle(iteminfo.name) + " [crafted by " + character.name + "]";
			endif
			upgradeitem.quality := upgradeitem.quality + 0.1;
			upgradeitem.hp := upgradeitem.maxhp;
			upgradeitem.buyprice := -1;
		else
			SubtractAmount (upgradematerial, RandomInt (CInt (materialneeded/2)) + 1 );
			if (RandomInt (10) == 0)
				SendSysMessage (character, "You ruin the item trying to upgrade it.");
				DestroyItem (upgradeitem);
				releaseitem(upgradematerial);
				return 0;
			else
				SendSysMessage (character, "You fail and destroy some " + materialinfo[1]);
			endif
		endif
	endwhile
	releaseitem(upgradematerial);
	return 1;
endfunction



// Repairs an item
function repairitem(character, item, repairmaterial, repairskill, sound, requiresanvil := 0)
	if (requiresanvil)
		if (!SearchForAnvilInArea (character))
			SendSysmessage (character, "You must be near an anvil to make anything.");
			return;
		endif
		
		if (!FindForgeInArea (character))
			SendSysmessage (character, "You must be near a forge to make anything.");
			return;
		endif
		
		if (!CheckSkillForDifferentMetals (character, repairmaterial.objtype, repairskill))
			sendsysmessage(character, "our skill is too low to work with that material!");
			return;
		endif
	endif
	
	if(GetObjproperty(item, "etherial"))
		SendSysmessage (character, "You can not even begin to understand how to this item works!");
		return;
	endif
	
	if (item.hp == item.maxhp)
		sendsysmessage (character, "That is already in full repair");
		return;
	endif
	
	if (!item.maxhp or !item.quality)
		SendSysmessage (character, "That item cannot be repaired.");
		return;
	endif

	if (item.name["bone"] or item.name["Bone"])
		SendSysmessage (character, "You can not repair bone!");
		return;
	endif

	//make sure the item is good enough to be repairable
	if (item.hp <= 0)
		SendSysmessage (character, "The brittle material breaks when you handle it.");
		DestroyItem (item);
		return;
	endif

	//setup some variables
	var item_damaged_amount := item.maxhp - item.hp;

	//now determine how many ingots and how much skill it will take to repair this item
	var materialneeded := CINT (item_damaged_amount/20);
	if (!materialneeded)
		materialneeded := 1;
	endif

	if (!Accessible (character, repairmaterial))
		SendSysMessage (character, "You can't reach that.");
		return;
	endif
	
	if (repairmaterial.amount < materialneeded)
		SendSysMessage (character, "You need " + materialneeded + " in raw materials to do the repair");
		return;
	endif
	
	if (!ReserveItem (repairmaterial))
		SendSysMessage (character, "You cannot use that right now.");
		return;
	endif

	PlaySoundEffect(character, sound);
	sleep (3);
	PlaySoundEffect(character, sound);
	sleep (3);
	PlaySoundEffect(character, sound);
	sleep (3);

	SubtractAmount (repairmaterial, materialneeded);
	
	var total_damage := GetObjProperty (item, "total_damage");
	if (!total_damage)
		total_damage := 0;
	endif

	var times_repaired := GetObjProperty (item, "times_repair");
	if (!times_repaired)
		times_repaired := 0;
	endif

	var maxtimes := GetObjProperty (item, "maxtimes_repair");
	if (!maxtimes)
		if (item.name["fine"] or item.name["Fine"] or item.name["efense"]) 
			maxtimes := 1; 
		elseif (item.name["urable"] or item.name["uarding"]) 
			maxtimes := 1;
		elseif (item.name["ugged"] or item.name["rotection"]) 
			maxtimes := 2;
		elseif (item.name["empered"] or item.name["ortification"]) 
			maxtimes := 2;
		elseif (item.name["ndestructable"] or item.name["nvulnerability"]) 
			maxtimes := 3;
		elseif (item.name["egacy"] or item.name["nvincibility"]) 
			maxtimes := 3;
		else
			maxtimes := 0;
		endif 
		SetObjProperty (item, "maxtimes_repair", maxtimes);
	endif

	var reset := 0;

	if (total_damage + item_damaged_amount > item.maxhp)
		if (times_repaired >= maxtimes)
			DestroyItem (item);
			SendSysmessage (character, "The brittle material breaks when you handle it.");
			releaseitem(repairmaterial);
			return;
		endif
		times_repaired := times_repaired +1;
		total_damage := 0;
		reset := 1;
	endif

	if (!CheckSkill (character, GetSkillIDByAttributeID(repairskill), -1, 0) )
		total_damage := total_damage + RandomInt (10);
		SetObjProperty (item, "total_damage", total_damage);
		if (total_damage + item_damaged_amount > item.maxhp)
			DestroyItem (item);
			SendSysmessage (character, "The brittle material breaks when you handle it.");
			releaseitem(repairmaterial);
			return;
		endif
		SendSysMessage (character, "You only manage to damage the item further...");
		releaseitem(repairmaterial);
		return;
	endif

	var skill_bonus := CINT (GetAttribute (character, GetAttributeIDBySkillID(repairskill))/4);
	//removed the first letter just in case it's case sensitive
	if (item.name["fine"] or item.name["Fine"] or item.name["efense"]) 
		skill_bonus := skill_bonus + 5; 
	elseif (item.name["urable"] or item.name["uarding"]) 
		skill_bonus := skill_bonus + 10; 
	elseif (item.name["ugged"] or item.name["rotection"]) 
		skill_bonus := skill_bonus + 15; 
	elseif (item.name["empered"] or item.name["ortification"]) 
		skill_bonus := skill_bonus + 20; 
	elseif (item.name["ndestructable"] or item.name["nvulnerability"]) 
		skill_bonus := skill_bonus + 25; 
	elseif (item.name["egacy"] or item.name["nvincibility"]) 
		skill_bonus := skill_bonus + 30; 
	endif 

	item_damaged_amount := CINT (item_damaged_amount * (100-skill_bonus)/100);
	if (item_damaged_amount < 5)
		item_damaged_amount := 5;
	endif

	if (reset == 0)
		total_damage := total_damage + item_damaged_amount;
	endif	
	
	SetObjProperty (item, "total_damage", total_damage);
	SetObjProperty (item, "times_repair", times_repaired);
	item.hp := item.maxhp;
	SendSysMessage (character, "You repair the item completely");
	releaseitem(repairmaterial);
	return;
endfunction



// Laminate an item
function laminateitem(character, item, material, cfg_file, repairskill, sound, requiresanvil := 0 )
	if (requiresanvil)
		if (!SearchForAnvilInArea (character))
			SendSysmessage (character, "You must be near an anvil to modify items.");
			return;
		endif
		
		if (!FindForgeInArea (character))
			SendSysmessage (character, "You must be near a forge to modify items.");
			return;
		endif
		
		if (!CheckSkillForDifferentMetals (character, material.objtype, repairskill))
			sendsysmessage(character, "Your skill is too low to work with that material!");
			return;
		endif
	endif

	if (!IsMagicalItem (item))
		sendsysmessage (character, "You can only color magical items");
		return;
	endif
	
	if (item.hp < item.maxhp)
			sendsysmessage (character, "The item must be in a state of full repair first.");
			return;
	endif	
	
	var elem := FindConfigElem (cfg_file, item.objtype);
	if (!elem)
		SendSysMessage (character, "Invalid item type for this tool.");
		return;
	endif
	
	var materialneeded := cint (elem.material/2);
	var time_delay := elem.time;
	if (time_delay > 1)
		time_delay := time_delay - 1;
	endif;
	
	SendSysMessage (character, "Laminating that item will take " + materialneeded + " material.");

	if (GetAttribute (character, repairskill) < 90)
		SendSysMessage (character, "Your skill is too low to laminate that item.");
		return;
	endif
	
	if (!Accessible (character, material))
		SendSysMessage (character, "You can't reach that.");
		return;
	endif
	
	if (!ReserveItem (material))
		SendSysMessage (character, "You cannot use that right now.");
		return;
	endif
	
	if (material.amount < materialneeded)
		SendSysMessage (character, "You need " + materialneeded + " ingots.  That's only " + material.amount + "!");
		return;
	endif
	
	for i := 1 to time_delay
		i := i;  // just to make the warning go away
		PlaySoundEffect(character, sound);
		sleep (2);
	endfor
	PlaySoundEffect(character, sound);
	
	if (RandomInt (1000) == 0)
		SendSysMessage (character, "You fail, destroying the item.");
		DestroyItem (item);
		releaseitem(material);
		return;
	elseif ( RandomInt (100) < 10 )
		SendSysMessage (character, "You fail, destroying some ingots.");
		SubtractAmount (material, RandomInt (materialneeded));
		releaseitem(material);
		return;
	else
		SendSysMessage (character, "You succeed in laminating the item.");
		item.color := material.color;
		SubtractAmount (material, materialneeded);
		releaseitem(material);
		return;
	endif
endfunction


///////////////////
//  Searches the area around the character for an anvil
///////////////////

function SearchForAnvilInArea (byref character)
	foreach item in ListItemsNearLocation (character.x, character.y, character.z, 2)
		if ((item.objtype == UOBJ_ANVIL1) or (item.objtype == UOBJ_ANVIL2))
			return 1;
		endif
	endforeach

	if (seekphantomforge(character,2,PF_ANVIL))
		return 1;
	endif

	return 0;
endfunction

///////////////////
//  Searches the area around the character for a forge
///////////////////

function FindForgeInArea (character)
	foreach item in ListItemsNearLocation(character.x, character.y, character.z,2)
		if ( item.objtype >= UOBJ_FORGE_START and item.objtype <= UOBJ_FORGE_END )
			return 1;
		elseif (item.objtype == UOBJ_SMALL_FORGE)
			return 1;
		endif
	endforeach
	
	if (seekphantomforge(character,2,PF_FORGE))
		return 1;
	endif
	
	return 0;
endfunction


///////////////////
//  Determines the minimum skill for working with a particular metal
///////////////////

function CheckSkillForDifferentMetals (character, ingot, SKILLID)

	var character_skill := GetAttribute (character, SKILLID);
		
	case ( ingot )
		UOBJ_IRON_INGOT:
			return 1;   //anyone can work with iron
		UOBJ_BRONZE_INGOT:
			if (character_skill > 60)
				return 1;
			endif
		UOBJ_DULLCOPPER_INGOT:
			if (character_skill > 70)
				return 1;
			endif
		UOBJ_COPPER_INGOT:
			if (character_skill > 80)
				return 1;
			endif
		UOBJ_ONYX_INGOT:
			if (character_skill > 85)
				return 1;
			endif
 		UOBJ_PYRITE_INGOT:
			if (character_skill > 90)
				return 1;
			endif
		UOBJ_MALACHITE_INGOT:
			if (character_skill > 95)
				return 1;
			endif
 		UOBJ_AZURITE_INGOT:
			if (character_skill > 98)
				return 1;
			endif
		UOBJ_BLUESTEEL_INGOT:
			if (character_skill > 98)
				return 1;
			endif
		UOBJ_RUBYSTEEL_INGOT:
			if (character_skill > 98)
				return 1;
			endif
	endcase
	if (character_skill > 98)
		return 1;
	endif
endfunction

function Isfancyore( item )
	if (item.objtype >= 0x6012 and item.objtype <=  0x7064)
		return 1;
	elseif (item.objtype >= 0x8019 and item.objtype <=  0x8031)
		return 1;
	else
		return 0;
	endif
endfunction


///////////////////
//  determines if the given item is a type of hide
///////////////////

function IsTypeOfHide (item)

	case (item.objtype)
		UOBJ_HIDE:
		UOBJ_BROWN_HIDE:
		UOBJ_BLUE_HIDE:
		UOBJ_RED_HIDE:
		UOBJ_GREEN_HIDE:
		UOBJ_SILVER_HIDE:
			return 1;
	endcase

	return 0;

endfunction


// Splits the material/component <type> <amount> from the cfg file into string<type> and int<amount>
function splitcomponent(component)
	var makematerial := splitwords(component);
	makematerial[2] := cint(makematerial[2]);
	return makematerial;
endfunction


///////////////////
//  Determines if the item objtype passed is a hat
///////////////////

function IsAHat (item)

	case (item.objtype)
		0x153F:
		0x1543:
		0x1713:
		0x1714:
		0x1715:
		0x1716:
		0x1717:
		0x1718:
		0x1719:
		0x171A:
		0x171B:
		0x171C:
		0x141b:	//orc mask
		0x1545:	//bear mask
		0x1547:	//deer mask
		0x1549:	//tribal mask
		0x154b:	//tribal mask
			return 1;
	endcase

	return 0;

endfunction

///////////////////
//  Determines if the item objtype passed is a helm
///////////////////

function IsAHelm (item)

	case (item.objtype)
		0x1408:
		0x1409:
		0x140A:
		0x140B:
		0x140C:
		0x140D:
		0x140E:
		0x140F:
		0x1412:
		0x1419:
		0x1DB9:
		0x708b:
		0x13BB:
		0x13C0:
		0x708c:
		0x1451:
			return 1;
	endcase

	return 0;

endfunction