include "include/spawn";
include "include/poison";
include "../pkg/npcs/townfolk/guards/henchman";
include "../pkg/tools/random/random";
include "../pkg/perks/perk";
include "../pkg/perks/include/perkcredit";

///////////////////
//  tries to heal the patient
///////////////////

function HealPatient (healer, patient, bandages)
	if (Distance (healer, patient) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var healer_healing := GetAttribute (healer, ATTRIBUTEID_HEALING);
	var ignore_range := healer_healing/20;
	var healer_anatomy := GetAttribute (healer, ATTRIBUTEID_ANATOMY);
	var bandaging_time := 20 - CINT(healer_anatomy/15);
	var heal_dice := DetermineHealDice (healer_healing, healer_anatomy);

	SendSysmessage (healer, "You begin applying a bandage...", FONT_NORMAL, 53);
	if (patient != healer)
		SendSysMessage (patient, healer.name + " begins applying a bandage",
			FONT_NORMAL, 53);
		bandaging_time := bandaging_time - 5;
	endif

	var DoneHealing := ReadGameClock() + bandaging_time;
	SetObjProperty (healer, "#heal", DoneHealing);
	detach();

	var starthp := GetVital (healer, "Life");
	var healer_dexterity := GetAttribute (healer, "Dexterity");
	var difficulty_mod := 0;

	var slipsetting := GetObjProperty( healer, "autoheal_options");

	repeat
		if (GetVital (healer, "Life") < CINT((starthp*(100-ignore_range))/100))
			//try to save based on dex
			if (random (100) > CINT (healer_dexterity/2) )
				if ( slipsetting[7] == 1 || !slipsetting[7]); // don't ask, just have faith
					SendSysMessage (healer, "Your fingers slip!", FONT_NORMAL, MESSAGE_COLOR_WARNING);
				endif
				difficulty_mod := difficulty_mod + 5;
				starthp := GetVital (healer, "Life");
			endif
		endif
		if (!patient or !healer)
			SendSysmessage (healer, "Now where did he go...", FONT_NORMAL, MESSAGE_COLOR_BAD);
			EraseObjProperty (healer, "#heal");
			return;
		elseif (patient.dead)
			SendSysMessage (healer, "You need to be a bit faster, next time.", FONT_NORMAL, MESSAGE_COLOR_BAD);
			EraseObjProperty (healer, "#heal");
			return;
		endif
		sleepms(250);
	until (ReadGameClock() >= DoneHealing);

	if (Distance (healer, patient) > 2)
		SendSysmessage (healer, patient+" is too far away to heal!", FONT_NORMAL, MESSAGE_COLOR_BAD);
		if (healer.npctemplate[":guards:"])
			PrintTextAbove (healer, "Argh, "+patient.name+" is too far away to heal!");
		endif
		EraseObjProperty (healer, "#heal");
		return;
	endif

	if (GetVital (patient, "Life") >= GetVitalMaximumValue (patient, "Life"))
		SendSysMessage (healer, "The patient seems to be quite all right.", FONT_NORMAL, MESSAGE_COLOR_GOOD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var difficulty := GetVitalMaximumValue (patient, "Life") - GetVital (patient, "Life");
	difficulty := CINT (difficulty/100);
	if (difficulty > 100)
		difficulty := 100;
	endif

	difficulty := difficulty + difficulty_mod;
	SubtractAmount (bandages, 1);

	var waspoisoned := 0;
	if (patient.poisoned or GetObjProperty(patient, "iampoisoned"))
		var poison_difficulty := 50 + GetPoisonLevel (patient)*5;
		if (CheckSkill (healer, SKILLID_HEALING, poison_difficulty, 0) )
			if (healer != patient)
				SendSysMessage (healer, "You have cured " + patient.name, FONT_NORMAL, MESSAGE_COLOR_GOOD);
				SendSysMessage (patient, healer.name + " has cured you!", FONT_NORMAL, MESSAGE_COLOR_GOOD);
			else
				SendSysMessage(healer,"You have cured yourself!", FONT_NORMAL, MESSAGE_COLOR_GOOD);
			endif
			CurePoison (patient, 0, 0);
			perknotehelp(healer,patient);
			EraseObjProperty(patient, "iampoisoned");
			waspoisoned := 1;
		else
			if (healer != patient)
				SendSysMessage (healer, "You have FAILED to cure " + patient.name, FONT_NORMAL, MESSAGE_COLOR_BAD);
				SendSysMessage (patient, healer.name + " has FAILED to cure you!", FONT_NORMAL, MESSAGE_COLOR_BAD);
			else
				SendSysMessage (healer,"You have FAILED to cure yourself!", FONT_NORMAL, MESSAGE_COLOR_BAD);
			endif
		endif		
	endif

	if (CheckSkill (healer, SKILLID_HEALING, difficulty, 0) )
		var points_healed := RandomDiceStr (heal_dice);
		if (waspoisoned)
			points_healed := CINT (points_healed * 2 / 3);
		endif
		var patient_wounds := GetVitalMaximumValue (patient, "Life") - GetVital (patient, "Life");
		patient_wounds := CINT (patient_wounds/100);
		if (points_healed > patient_wounds)
			points_healed := patient_wounds;
		endif
		
		HealDamage (patient, points_healed);
		perknotehelp(healer,patient);
		SendSysMessage (healer, "You have healed " + points_healed + " hit points.", FONT_NORMAL, MESSAGE_COLOR_GOOD);
		if (patient != healer)
			SendSysMessage (patient, healer.name + " has healed " + points_healed + " hit points.", FONT_NORMAL, MESSAGE_COLOR_GOOD);
		endif
	else
		SendSysMessage (healer, "You were unsuccessful at binding the wound.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		if (patient != healer)
			SendSysMessage (patient, healer.name + " was unable to bind your wound.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		endif
	endif
	EraseObjProperty (healer, "#heal");
endfunction




///////////////////
//  Attempts to resurrect the chosen player
///////////////////

function ResurrectPlayer (healer, patient, bandages)
	var healer_healing := GetAttribute (healer, ATTRIBUTEID_HEALING);
	var healer_anatomy := GetAttribute (healer, ATTRIBUTEID_ANATOMY);
	var res_chance := DetermineResChance (healer_healing, healer_anatomy);
	if ( !res_chance )
		SendSysMessage (healer, "You lack the skill to resurrect the dead.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	if (GetObjProperty (patient, "cut"))
		SendSysmessage (healer, "You can't resurrect a corpse that has lost its head." );
		return;
	endif
	var npc := SystemFindObjectBySerial(getobjproperty(patient,"ghost_serial"));
	if (!npc)
		npc := SystemFindObjectBySerial (getobjproperty (patient,"ghost_serial"),  SYSFIND_SEARCH_OFFLINE_MOBILES);
		if (!npc)
			SendSysMessage(healer,"The ghost seems to have moved on...");
			return;
		endif
	endif
	if (!npc.dead)
		SendSysMessage(healer,"The spirit seems to have moved on...");
		return;
	endif
	if (Distance (healer, patient) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SendSysMessage (healer, "You begin applying a bandage...");
	SendSysMessage (patient, healer.name + " attempts to resurrect you...");
	var wait_time := 15 - CINT (healer_anatomy/20);
	sleep (wait_time);

	if (Distance (healer, patient) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SubtractAmount (bandages, 1);
	

	if (random (100) <= res_chance)
		MoveObjectToLocation(npc, patient.x, patient.y, patient.z, realm := _DEFAULT_REALM, flags := MOVEOBJECT_FORCELOCATION );
		Resurrect (npc);
		SetVital (npc, "Life", CINT(GetVitalMaximumValue(npc, "Life")/2));
		SetVital (npc, "Statima", CINT(GetVitalMaximumValue(npc, "Statima")/2));
		SetVital (npc, "Mana", CINT(GetVitalMaximumValue(npc, "Mana")/2));
		var robe := GetEquipmentByLayer(npc,LAYER_ROBE);
		set_critical(1);
		DestroyItem( robe );
		set_critical(0);
		foreach item in ListRootItemsInContainer( patient )
			if ( !EquipItem( npc, item ) )
				MoveItemToContainer( item, npc.backpack );
			endif
		endforeach
		DestroyItem( patient );
		
		perknotehelp(healer,patient);
		AwardVirtueForRes (healer,patient);
		SubtractAmount (bandages, 1);
	else
		SendSysmessage (healer, "You were unsuccessful at returning the patient to this world.", FONT_NORMAL, MESSAGE_COLOR_BAD);
	endif
	EraseObjProperty (healer, "#heal");
endfunction




///////////////////
//  Attempts to resurrect the chosen NPC based on the corpse
///////////////////

function TryToResurrectHenchman (healer, corpse, bandages)
	var master_serial := GetObjProperty (corpse, "master");
	if (!master_serial)
		SendSysMessage (healer, "You can't resurrect that!");
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var mymaster := SystemFindObjectBySerial (master_serial, SYSFIND_SEARCH_OFFLINE_MOBILES );
	if (!mymaster)
		set_critical(0);
		return 0;
	endif

	var healer_healing := GetAttribute (healer, ATTRIBUTEID_HEALING);
	var healer_anatomy := GetAttribute (healer, ATTRIBUTEID_ANATOMY);
	var res_chance := DetermineResChance (healer_healing, healer_anatomy);
	if (!res_chance)
		SendSysMessage (healer, "You lack the skill to resurrect the dead.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	if (Distance (healer, corpse) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SendSysMessage (healer, "You begin applying a bandage...");
	var wait_time := 15 - CINT (healer_anatomy/20);
	sleep (wait_time);

	if (Distance (healer, corpse) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SubtractAmount (bandages, 1);

	if (random (100) <= res_chance)
		if (!ResurrectHenchman (corpse))
			SendSysMessage (healer, "You cannot resurrect the patient");
		endif
		
	else
		SendSysmessage (healer, "You were unsuccessful at returning the patient to this world.", FONT_NORMAL, MESSAGE_COLOR_BAD);
 	endif
	EraseObjProperty (healer, "#heal");
endfunction

///////////////////
//  Attempts to resurrect the warrior
///////////////////

function TrytoResurrectTownguard (healer, corpse, bandages)
	set_critical (1);
	var master_serial := GetObjProperty (corpse, "master");
	if (!master_serial)
		set_critical (0);
		return 0;
	endif

	var mymaster := SystemFindObjectBySerial (master_serial, SYSFIND_SEARCH_OFFLINE_MOBILES );
	if (!mymaster)
		set_critical(0);
		return 0;
	endif

	var mynpctemplate := GetObjProperty (corpse, "npctemplate");
	if (!mynpctemplate)
		set_critical(0);
		SendSysMessage (mymaster, "unknown template " + CSTR(mynpctemplate) + ", can't res");
		return 0;
	endif

	var healer_healing := GetAttribute (healer, ATTRIBUTEID_HEALING);
	var healer_anatomy := GetAttribute (healer, ATTRIBUTEID_ANATOMY);
	var res_chance := DetermineResChance (healer_healing, healer_anatomy);
	if (!res_chance)
		SendSysMessage (healer, "You lack the skill to resurrect the dead.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	if (Distance (healer, corpse) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SendSysMessage (healer, "You begin applying a bandage...");
	var wait_time := 15 - CINT (healer_anatomy/20);
	sleep (wait_time);

	if (Distance (healer, corpse) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SubtractAmount (bandages, 1);	

if (random (100) <= res_chance)
	var new_npc := CreateNpcFromTemplate (mynpctemplate, corpse.x, corpse.y, corpse.z);
	if (!new_npc)
		set_critical(0);
		return 0;
	endif

	perknotehelp(healer,new_npc);

	SetObjProperty (new_npc, "master", master_serial);
	new_npc.hp := 1;
	if (GetObjProperty (corpse, "gender") )
		new_npc.gender := GetObjProperty (corpse, "gender");
		new_npc.graphic := 400 + new_npc.gender;
	endif
	
	if (GetObjProperty (corpse, "g") )
		SetObjProperty (new_npc, "g", GetObjProperty (corpse, "g"));
	endif
		
	new_npc.color := GetObjProperty (corpse, "color");
	SetObjProperty (new_npc, "gender", new_npc.gender);
	SetObjProperty (new_npc, "color", new_npc.color);
	var temp_string := corpse.desc;
	temp_string["A corpse of "] := "";
	new_npc.name := temp_string;

	var newbackpack := CreateItemAtLocation (5288 , 1176 , 0, UOBJ_BACKPACK, 1);
	if (!EquipItem (new_npc, newbackpack))
		DestroyItem (newbackpack);
	endif

	foreach item in ListRootItemsInContainer (corpse)
		if (!EquipItem (new_npc, item))
			MoveItemToContainer (item, new_npc.backpack);
		endif
	endforeach

	if (GetObjProperty (corpse, "hairstyle"))
		var hairstyle := GetObjProperty (corpse, "hairstyle");
		if (hairstyle[1])
			var hair := CreateItemAtLocation (5288 , 1176 , 0, CINT (hairstyle[1]), 1);
			hair.color := hairstyle[2];
			if (!EquipItem (new_npc, hair))
				DestroyItem (hair);
			endif
		endif
		if (hairstyle[3])
			var beard := CreateItemAtLocation (5288 , 1176 , 0, CINT (hairstyle[3]), 1);
			beard.color := hairstyle[4];
			if (!EquipItem (new_npc, beard))
				DestroyItem (beard);
			endif
		endif
	endif

	DestroyItem (corpse);
	set_critical (0);
	return 1;

	else
		SendSysmessage (healer, "You were unsuccessful at returning the patient to this world.", FONT_NORMAL, MESSAGE_COLOR_BAD);
 	endif
	EraseObjProperty (healer, "#heal");
endfunction







///////////////////
//  Attempts to resurrect the chosen NPC based on the corpse
///////////////////

function ResurrectPet (healer, corpse, bandages)
	var npctemplate := GetObjProperty (corpse, "npctemplate");
	if (!npctemplate)
		SendSysMessage (healer, "You can't resurrect that!");
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var master_serial := GetObjProperty (corpse, "master");
	if (!master_serial)
		SendSysMessage (healer, "You can't resurrect that!");
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var master := SystemFindObjectBySerial (master_serial);
	if (!master)
		SendSysMessage (healer, "You can't resurrect that!");
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var healer_healing := GetAttribute (healer, ATTRIBUTEID_HEALING);
	var healer_anatomy := GetAttribute (healer, ATTRIBUTEID_ANATOMY);
	var res_chance := DetermineResChance (healer_healing, healer_anatomy);
	if (!res_chance)
		SendSysMessage (healer, "You lack the skill to resurrect the dead.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	if (Distance (healer, corpse) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SendSysMessage (healer, "You begin applying a bandage...");
	var wait_time := 15 - CInt (healer_anatomy/20);
	sleep (wait_time);

	if (Distance (healer, corpse) > 2)
		SendSysMessage (healer, "The patient is too far away.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	SubtractAmount (bandages, 1);
	if (! (random (100) <= res_chance))
		SendSysmessage (healer, "You were unsuccessful at returning the patient to this world.", FONT_NORMAL, MESSAGE_COLOR_BAD);
		EraseObjProperty (healer, "#heal");
		return;
	endif

	var rescount := GetObjProperty (corpse, "rescount");
	if (!rescount)
		rescount := 0;
	endif

	if (rescount * 10 > GetAttribute (master, ATTRIBUTEID_TAMING))
		SendSysMessage (healer, "It seems to have gone to the big pet shop in the sky...");
		EraseObjProperty (healer, "#heal");
		return;
	endif

	set_critical (1);
	var new_npc := SpawnNPCAtLocation (npctemplate, corpse.x, corpse.y, corpse.z);
	if (!new_npc)
		SendSysMessage (healer, "It seems to have gone to the big pet shop in the sky...");
		EraseObjProperty (healer, "#heal");
		return;
	endif

	perknotehelp(healer,new_npc);
	var newbackpack := CreateItemAtLocation (5288 , 1176 , 0, UOBJ_BACKPACK, 1);
	if (!EquipItem (new_npc, newbackpack))
		DestroyItem (newbackpack);
	endif

	foreach item in ListRootItemsInContainer (corpse)
		if (!equipItem (new_npc, item))
			MoveItemToContainer (item, new_npc.backpack);
		endif
	endforeach

	if (GetObjProperty (corpse, "color") )
		new_npc.color := GetObjProperty (corpse, "color");
		SetObjProperty (new_npc, "color", GetObjProperty (corpse, "color"));
	endif

	SetObjProperty (new_npc, "master", master.serial);
	new_npc.script := ":friendlies:tamed";
	RestartScript (new_npc);
	SetVital (new_npc, "Life", 100);

	var temp_string := corpse.desc;
	temp_string["A corpse of "] := "";
	new_npc.name := temp_string;
	SetObjProperty (new_npc, "rescount", rescount + 1);
	SetObjProperty (new_npc, "pet", 1);

	DestroyItem (corpse);
	set_critical (0);
	EraseObjProperty (healer, "#heal");
endfunction


///////////////////
//  determines the healers chance of successfully resurrecting the patient, based on their
//  healing and anatomy skill
///////////////////

function DetermineResChance (byref healer_healing, byref healer_anatomy)
	var res_chance := 0;
	var skill_level := CINT (healer_healing/10 + healer_anatomy/20);
	if (skill_level > 19)
		skill_level := 19;
	endif

	case (skill_level)
		6:
			res_chance := 5;
		7:
			res_chance := 10;
		8:
			res_chance := 15;
		9:
			res_chance := 20;
		10:
			res_chance := 25;
		11:
			res_chance := 30;
		12:
			res_chance := 35;
		13:
			res_chance := 40;
		14:
			res_chance := 50;
		15:
			res_chance := 60;
		16:
			res_chance := 61;
		17:
			res_chance := 62;
		18:
			res_chance := 63;
		19:
			res_chance := 64;
		default:
			res_chance := 0;
	endcase;

	return res_chance;
endfunction




///////////////////
//  determines the healers chance of successfully resurrecting the patient, based on their
//  healing and anatomy skill
///////////////////

function DetermineHealDice (byref healer_healing, byref healer_anatomy)
	var heal_dice := "1d5";
	var skill_level := CINT (healer_healing/10) + CINT(healer_anatomy/20);
	if (skill_level > 19)
		skill_level := 19;
	endif

	case (skill_level)
		1:
			heal_dice := "2d10";
		2:
			heal_dice := "2d10";
		3:
			heal_dice := "2d10";
		4:
			heal_dice := "2d10";
		5:
			heal_dice := "3d10";
		6:
			heal_dice := "4d10";
		7:
			heal_dice := "5d10";
		8:
			heal_dice := "6d10";
		9:
			heal_dice := "7d10";
		10:
			heal_dice := "8d10";
		11:
			heal_dice := "9d10";
		12:
			heal_dice := "9d10";
		13:
			heal_dice := "10d10";
		14:
			heal_dice := "10d10";
		15:
			heal_dice := "11d10";
		16:
			heal_dice := "11d10+1";
		17:
			heal_dice := "11d10+2";
		18:
			heal_dice := "11d10+3";
		19:
			heal_dice := "11d10+4";
		default:
			heal_dice := "1d5";
	endcase;

	return heal_dice;
endfunction
