use guilds;

///////////////////
//  Search for the nearest stone from the mobile
///////////////////

function FindMyStone (me)
	var mystone := FindStoneByLocation (me.x, me.y);
	return mystone;
endfunction




///////////////////
//  Search for the nearest stone to the location
///////////////////

function FindStoneByLocation (x, y)
	var parms := { };
	parms[1] := x;
	parms[2] := y;
	var mystone := run_script_to_completion (":townstones:findstone", parms);
	return mystone;
endfunction




///////////////////
//  Finds the stone of the given townnumber (1-20)
///////////////////

function GetStoneByNumber (townnum)
	var stoneserial := GetGlobalProperty ("Townstone"+townnum);
	if (stoneserial)
		var thatstone := SystemFindObjectBySerial (stoneserial);
		if (thatstone)
			return thatstone;
		endif
	endif
	return 0;
endfunction




///////////////////
//  Determines if the given player is banned from the given townstone
///////////////////

function BannedFromTown (who, townnum)
	var stone := GetStoneByNumber (townnum);
	return BannedFromStone (who, stone);
endfunction

function BannedFromStone (who, stone)
	if (!stone or !who)
		return 0;
	endif

	if (who.criminal)
		return 1;
	endif

	var bans := GetObjProperty (stone, "bans");
	if (!bans)
		return 0;
	endif
	if (bans.exists (who.serial))
		SendSysMessage (who, "You have been banned from " + stone.name);
		RemoveFromRoster (who, GetObjProperty (stone, "townnum"));
		return 1;
	endif
	return 0;
endfunction



///////////////////
//  Determines if the given player is outcast from the given townstone
///////////////////

function OutcastFromTown (who, townnum)
	var stone := GetStoneByNumber (townnum);
	return OutcastFromStone (who, stone);
endfunction

function OutcastFromStone (who, stone)
	if (!stone or !who)
		return 0;
	endif

	var outcasts := GetObjProperty (stone, "outcasts");
	if (!outcasts)
		return 0;
	endif
	if (outcasts.exists (who.serial))
		SendSysMessage (who, "You have been cast out from " + stone.name);
		return 1;
	endif
	return 0;
endfunction




///////////////////
//  Removes the given player from the town roster
///////////////////

function RemoveFromRoster (who, townnum)
	if (!who or !townnum)
		return;
	endif
	
	var yourtown := GetObjProperty (who, "townnum");
	if (yourtown != townnum)
		return;
	endif
	
	var oldstone := GetStoneByNumber (townnum);
	if (!oldstone)
		return;
	endif

	set_critical(1);
	var roster := GetObjProperty (oldstone, "roster");
	if (roster.exists (who.serial))
		roster.erase (who.serial);
		SetObjProperty (oldstone, "roster", roster);
	endif
	
	var votes  := GetObjProperty (oldstone, "votes");
	if (votes.exists (who.serial))
		votes.erase (who.serial);
		SetObjProperty (oldstone, "votes", votes);
	endif

	var militia := GetObjProperty (oldstone, "militia");
	if (militia.exists (who.serial))
		militia.erase (who.serial);
		SetObjProperty (oldstone, "militia", militia);
	endif

	if (who.guildid)
		who.title_guild := "";
		var guild := FindGuild (who.guildid);
		if (guild)
			guild.removemember (who);
		endif
	endif

	who.title_race := "";
	EraseObjProperty (who, "townnum");
	EraseObjProperty (who, "town");
	set_critical(0);
endfunction




///////////////////
//  Figures out the name of the town from the townnum
///////////////////

function GetTownName (townnum)
	var stone := GetStoneByNumber (townnum);
	if (!stone)
		return "";
	endif
	if (GetObjProperty (stone, "town"))
		return GetObjProperty (stone, "town");
	endif
	return "";
endfunction




///////////////////
//  Gets the name of the empire
///////////////////

function GetEmpireName (townnum)
	var masterstone := GetEmpireMasterStone (townnum);
	if (!masterstone)
		return "";
	endif

	var EmpireName := GetObjProperty (masterstone, "empirename");
	if (!EmpireName)
//		EmpireName := GetObjProperty (masterstone, "town") + " Militia";
		EmpireName := "No Empire";
	endif

	return EmpireName;
endfunction




///////////////////
//  Gets the townnumber of the empire
///////////////////

function GetEmpireNumber (townnum)
	var masterstone := GetEmpireMasterStone (townnum);
	if (!masterstone)
		return 0;
	endif

	var EmpireNum := GetObjProperty (masterstone, "townnum");
	if (!EmpireNum)
		return 0;
	endif
	return EmpireNum;
endfunction



///////////////////
//  Figures out the highest level stone of an empire
///////////////////

function GetEmpireMasterStone (townnum)
	var stone := GetStoneByNumber (townnum);
	if (!stone)
		return 0;
	endif

	if (GetObjProperty (stone, "fealty"))
		stone := GetStoneByNumber (GetObjProperty (stone, "fealty"));
	endif
	return stone;
endfunction




///////////////////
//  Returns an array of all the stonenumbers of stones loyal to the given empire
///////////////////

function GetEmpireSubStones (empirenum)
	var townstones := {};
	for i := 1 to 20
		var substone := GetStoneByNumber (i);
		if (substone)
			var fealty := GetObjProperty (substone, "fealty");
			if (fealty and fealty == empirenum)
				townstones.append (substone);
			endif		
		endif
	endfor
	return townstones;
endfunction





///////////////////
//  Tries to find the mayor of the given town
///////////////////

function FindMayorOfTown (townnum)
	var stone := GetStoneByNumber (townnum);
	if (!stone)
		return 0;
	endif
	
	var votes := GetObjProperty (stone, "votes");	//dictionary
	if (!votes)
		return 0;
	endif

	var roster := GetObjProperty (stone, "roster");	//dictionary
	if (!roster)
		return;
	endif

	var mayorvotes := dictionary;
	foreach playerserial in (votes.keys())
		var player := FindPlayerBySerial (playerserial);
		if (player)
			var laston := GetObjProperty (player, "logontime");
			if (!laston)
				laston := 0;
			endif
			if (laston + 1209600 > ReadGameClock())
				if (mayorvotes.exists (votes[playerserial]))
					mayorvotes [votes [playerserial]] := mayorvotes [votes [playerserial]] + 1;
				else
					mayorvotes [votes [playerserial]] := 1;
				endif
			endif
		endif
	endforeach
	
	//make sure somebody voted
	if (!mayorvotes.keys())
		return 0;
	endif
	
	var chosenmayor := 0;
	var highestvote := 0;
	foreach key in (mayorvotes.keys())
//		AlertGM (key + " got " + mayorvotes[key] + " votes");
		if (mayorvotes[key] > highestvote)
			highestvote := mayorvotes[key];
			chosenmayor := key;
		endif
	endforeach
	
	//Shouldn't happend, but may as well make sure
	if (!highestvote or !chosenmayor)
		return 0;
	endif
	return FindPlayerBySerial (chosenmayor);
endfunction




///////////////////
//  Figures out the town's guild
///////////////////

function GetGuildByTownNum (townnum)
	var stone := GetEmpireMasterStone (townnum);
	if (!stone)
		return 0;
	endif
	
	if (GetObjProperty (stone, "guildid"))
		var guild := FindGuild (GetObjProperty (stone, "guildid"));
		if (guild)
			return guild;
		endif
	endif

	var guild := CreateGuild();
	SetObjProperty (stone, "guildid", guild.guildid);

	//declare war on everyone
	foreach otherguild in ListGuilds()
		guild.addenemyguild (otherguild);
	endforeach
	guild.addallyguild (guild);
	return guild;
endfunction




///////////////////
//  Finds the online or offline player by their serial number
///////////////////

function FindPlayerBySerial (plrserial)
	var plr := SystemFindObjectBySerial (plrserial);
	if (!plr)
		plr := SystemFindObjectBySerial (plrserial, SYSFIND_SEARCH_OFFLINE_MOBILES);
	endif
	if (plr)
		return plr;
	endif
	return 0;
endfunction







