///////////////////
//  New poison system, rewritten for POL 0.94.  Poison used to be a subset of the dotempmods functions
//  and relied on regen rates to work.  It now used the /pkg/skills/rogueskills/poisoning/poisondamage.src
//  script to do the damage.  This include function is the (hopefully) convenient front-end for the poison
//  system
//
//  Format for the #poisoninfo cprop:
//	[1] - poison level
//	[2] - time the poisoning should end
//	[3] - PID of poison damage script
///////////////////

function SetPoison (byref who, plvl, msg := "")
	//:poisoning:poisondamage is called, which does all the
	//work.  This is so we can tweak poisondamage
	//on-the-fly, and only have to recompile the one script

	var parms := {};
	parms[1] := who;
	parms[2] := plvl;
	parms[3] := msg;
	start_script (":poisoning:poisondamage", parms);
endfunction




///////////////////
//  Cures the poison
///////////////////

function CurePoison (byref who, sound := 0, msg := "")
	EraseObjProperty (who, "#poisoninfo");
	EraseObjProperty(who, "iampoisoned");
	who.poisoned := 0;
	if (sound)
		PlaySoundEffect (who, sound);
	endif
	if (msg)
		PrintTextAbovePrivate (who, msg, who);
	endif
endfunction




///////////////////
//  if we're currently poisoned, figures out how poisoned.
///////////////////

function GetPoisonLevel (byref who)
	var poisoned := GetObjProperty (who, "#poisoninfo");
	if (!poisoned)
		return 0;
	endif
	
	return (poisoned [1]);
endfunction

///////////////////
//  attempts to apply poison to the selected object
///////////////////

function ApplyPoisonToItem (byref character, byref the_poison)
	if (!ReserveItem (the_poison))
		return;
	endif

	SendSysmessage (character, "Select an object to which to apply this poison");
	var the_item := target (character);
	if (!the_item)
		SendSysmessage (character, "Targetting cancelled.");
		return;
	endif

	if (!Accessible (character, the_item))
		SendSysmessage (character, "You can't reach that!");
		return;
	endif

	var acfgfile := ReadConfigFile ("poisoning_items");
	var theitem := FindConfigElem (acfgfile, cint(the_item.objtype));   
	if (!theitem)
		SendSysmessage (character, "You can't put poison on that!");
		return;
	endif    
   
	var strength:= PoisonStr (the_poison);
	if (strength == 0)
		strength := 1;
	endif

	var skill := skillreq (strength);
	AddVirtue (character, -2);

	if (CheckSkill( character, SKILLID_POISONING, skill, 0 ))
		SetObjProperty (the_item, "poison_level", cstr(strength));
		SendSysmessage (character, "You succeed in poisoning the item.");
		the_item.hitscript := ":weapons:mainhit";
	else
		SendSysmessage (character, "You fail to poison the item."); 
	endif

	SubtractAmount (the_poison, 1);
	CreateIteminBackpack (character, UOBJ_EMPTY_BOTTLE, 1);   
endfunction




///////////////////
//  figures out the strength of the poison based on the potion type
///////////////////

function PoisonStr (byref the_poison)
	case (the_poison.objtype)
		UOBJ_LESSER_POISON_POTION:
			return 2;
		UOBJ_POISON_POTION:
			return 4;
		UOBJ_GREATER_POISON_POTION:
			return 7;
		UOBJ_DEADLY_POISON_POTION:
			return 10;
		default:
			return 8;
	endcase
endfunction




///////////////////
//  figures out the difficulty to use that kind of poison
///////////////////

function skillreq (str)
	case (str)
		1:
		2:
		3:	return 30;
		4:
		5:
		6:	return 55;
		7:
		8:	return 75;
		9:
		10:	return 95;
		default:
			return 90;
	endcase
endfunction

