use storage;

include "include/objtype";

function find_or_create_storage ( areaname )

	var area := FindStorageArea ( areaname );
	if (!area)
		area := CreateStorageArea ( areaname );
		if (area)
			print ( "Storage Area '" + areaname + "' created." );
		endif
	endif
	return area;

endfunction

function find_or_create_item ( storage, name, objtype )

	var item := FindRootItemInStorageArea ( storage, name );
	if (item)
		return item;
	endif

	//didn't find, so create
	item := CreateRootItemInStorageArea ( storage, name, objtype );
	return item;

endfunction

function IsLocationAccessible ( character, x, y, z )
    var xd, yd, zd;
    
    xd := character.x - x;
    if (xd < -1 or xd > 1) 
        return 0;
    endif
    
    yd := character.y - y;
    if (yd < -1 or yd > 1) 
        return 0;
    endif
    
    zd := character.z - z;
    if (zd < -10 or zd > 10)
        return 0;
    endif

    return CheckLosAt( character, x, y, z );

endfunction

//function min (x, y)
//	if (x<= y)
//		return x;
//	else
//		return y;
//	endif
//endfunction

function coordist ( x1 , y1 , x2 , y2 )

	var xd := x1 - x2;
	var yd := y1 - y2;
	if (xd < 0)
		xd := -xd;
	endif
	if (yd < 0)
		yd := -yd;
	endif
	if (xd > yd)
		return xd;
	else
		return yd;
	endif

endfunction

function AllocLockId()

	set_critical (1);
	var lockid := GetGlobalProperty( "nextlockid" );
	if (!lockid)
		lockid := 1;
	endif
	SetGlobalProperty ( "nextlockid", lockid+1 );

	set_critical (0);

	return lockid;
endfunction

function GetDirection (character, x, y)

var ns_desc := "";
	var ns_diff := 0;
	if (y < character.y)
        	ns_desc := "north";
        	ns_diff := character.y - y;
	elseif  (y > character.y)
		ns_desc := "south";
        	ns_diff := y - character.y;
	endif

	var ew_desc := "";
	var ew_diff := 0;
	if (x < character.x)
		ew_desc := "west";
		ew_diff := character.x - x;
	elseif  (x > character.x)
		ew_desc := "east";
		ew_diff := x - character.x;
	endif
	
	//Adjustments for the diagonals...
	if (ns_desc != "" and ew_desc != "")
		if (ns_diff > ew_diff * 2)
			ew_desc := "";
		elseif (ew_diff > ns_diff * 2)
			ns_desc := "";
		endif
	endif

	var total_desc := "";
	if (ns_desc == "" and ew_desc == "")
		total_desc := "nearby";
	else 
		total_desc :=ns_desc + ew_desc;
	endif

	return total_desc;

endfunction

///////////////////
//  dismounts the given mount
///////////////////

function Dismount (character, mount := 0)

	var parms := array;
	parms[1] := character;
	parms[2] := mount;
	return run_script_to_completion (":drocket:dismount", parms);

endfunction




///////////////////
//  opens the bank storage area
///////////////////

function OpenWorldBank()
	var bank := FindStorageArea ("World Bank");
	if (!bank)
		bank := CreateStorageArea ("World Bank");
	endif
	return bank;
endfunction




///////////////////
// finds the bankbox for the given character
///////////////////

function FindBankBox (acctname, nocreate := 0)

	var worldbank := OpenWorldBank();
	var bank_obj_name := CSTR ("Bankbox of " + acctname);
	var bankbox := FindRootItemInStorageArea (worldbank, bank_obj_name);
	if (!bankbox and !nocreate)
		bankbox := CreateRootItemInStorageArea( worldbank, bank_obj_name, UOBJ_BANKBOX );
	endif
	return bankbox;
endfunction


function OpenBrowser( who, url )
/*
0xA5 Packet
Open Web Browser (Variable # of bytes)
* BYTE cmd
* BYTE[2] blockSize
* BYTE[blockSize-3] null terminated full web address
*/

var tempvar := "";
var packetlength;
var i;

for( i := 1; i <= len( url ); i := i + 1 )
  tempvar := tempvar + ConvertLetterToHex( url[i] );
endfor

packetlength := (len( tempvar ) + 2 + 4) / 2;
packetlength := ConvertToPacketHex( packetlength );
CorrectSize( packetlength, 4 );

var terminator := cint(packetlength) - 3;

SendPacket(who, "A5"+packetlength+tempvar+terminator);
endfunction


function ConvertLetterToHex( letter )

   case( cstr(letter) )
     "A" : return "41";
     "�" : return "C4";
     "B" : return "42";
     "C" : return "43";
     "D" : return "44";
     "E" : return "45";
     "F" : return "46";
     "G" : return "47";
     "H" : return "48";
     "I" : return "49";
     "J" : return "4A";
     "K" : return "4B";
     "L" : return "4C";
     "M" : return "4D";
     "N" : return "4E";
     "O" : return "4F";
     "�" : return "D6";
     "P" : return "50";
     "Q" : return "51";
     "R" : return "52";
     "S" : return "53";
     "T" : return "54";
     "U" : return "55";
     "�" : return "DC";
     "V" : return "56";
     "W" : return "57";
     "X" : return "58";
     "Y" : return "59";
     "Z" : return "5A";

     "a" : return "61";
     "�" : return "E4";
     "b" : return "62";
     "c" : return "63";
     "d" : return "64";
     "e" : return "65";
     "f" : return "66";
     "g" : return "67";
     "h" : return "68";
     "i" : return "69";
     "j" : return "6A";
     "k" : return "6B";
     "l" : return "6C";
     "m" : return "6D";
     "n" : return "6E";
     "o" : return "6F";
     "�" : return "F6";
     "p" : return "70";
     "q" : return "71";
     "r" : return "72";
     "s" : return "73";
     "t" : return "74";
     "u" : return "75";
     "�" : return "FC";
     "v" : return "76";
     "w" : return "77";
     "x" : return "78";
     "y" : return "79";
     "z" : return "7A";

     "!" : return "21";
     "/" : return "2F";
     "[" : return "5B";
     "]" : return "5D";
     "@" : return "40";
     "=" : return "3D";
     "#" : return "23";
     "'" : return "27";
     "�" : return "B4";
     "`" : return "60";
     "{" : return "7B";
     "}" : return "7D";
     "_" : return "5F";
     "," : return "2C";
     "." : return "2E";
     "<" : return "3C";
     ">" : return "3E";
     "|" : return "7C";
     "^" : return "5E";
     "�" : return "DF";
     "?" : return "3F";
     ":" : return "3A";
     ";" : return "3B";
     "-" : return "2D";
     "+" : return "2B";
     "*" : return "2A";
     "(" : return "28";
     ")" : return "29";
     "1" : return "31";
     "2" : return "32";
     "3" : return "33";
     "4" : return "34";
     "5" : return "35";
     "6" : return "36";
     "7" : return "37";
     "8" : return "38";
     "9" : return "39";
     "0" : return "30";
     " " : return "20";
     default: exit;
  endcase

endfunction

function CorrectSize( byref pstring, laenge )

while( len(pstring) != laenge )
  pstring := "0"+pstring;
endwhile

endfunction


function ConvertToPacketHex( text )
var newhex := cstr(hex( text ));
newhex["0x"] := "";

return newhex;
endfunction

///////////////////
//  Sends messages to all online GMs.  Useful for important errors
///////////////////

function SendGMErrorMessage (text, syslogit := 0, mincmdlevel := 1)
	if (syslogit)
		syslog (text);
	endif

	syslog (text);
	foreach player in EnumerateOnlineCharacters ()
		if (player.cmdlevel)
			if (player.cmdlevel >= mincmdlevel)
				SendSysMessage (player, text);
			endif
		endif
	endforeach
endfunction


